
##############################
# Auto-generated. Do not Edit
##############################

import re
from pylipd.utils import uniqid
from pylipd.classes.archivetype import ArchiveType
from pylipd.classes.calibration import Calibration
from pylipd.classes.compilation import Compilation
from pylipd.classes.interpretation import Interpretation
from pylipd.classes.paleoproxy import PaleoProxy
from pylipd.classes.paleoproxygeneral import PaleoProxyGeneral
from pylipd.classes.paleounit import PaleoUnit
from pylipd.classes.paleovariable import PaleoVariable
from pylipd.classes.physicalsample import PhysicalSample
from pylipd.classes.resolution import Resolution

class Variable:
    """Auto-generated LinkedEarth class representing `Variable`."""
    def __init__(self):
        """Initialize a new Variable instance."""
        
        self.archiveType: ArchiveType = None
        self.calibratedVias: list[Calibration] = []
        self.columnNumber: int = None
        self.composite: bool = None
        self.description: str = None
        self.foundInDataset: None = None
        self.foundInTable: None = None
        self.instrument: None = None
        self.interpretations: list[Interpretation] = []
        self.maxValue: float = None
        self.meanValue: float = None
        self.medianValue: float = None
        self.minValue: float = None
        self.missingValue: str = None
        self.name: str = None
        self.notes: str = None
        self.partOfCompilations: list[Compilation] = []
        self.physicalSamples: list[PhysicalSample] = []
        self.primary: bool = None
        self.proxy: PaleoProxy = None
        self.proxyGeneral: PaleoProxyGeneral = None
        self.resolution: Resolution = None
        self.standardVariable: PaleoVariable = None
        self.uncertainty: str = None
        self.uncertaintyAnalytical: str = None
        self.uncertaintyReproducibility: str = None
        self.units: PaleoUnit = None
        self.values: str = None
        self.variableId: str = None
        self.variableType: str = None
        self.misc = {}
        self.ontns = "http://linked.earth/ontology#"
        self.ns = "http://linked.earth/lipd"
        self.type = "http://linked.earth/ontology#Variable"
        self.id = self.ns + "/" + uniqid("Variable.")

    @staticmethod
    def from_data(id, data) -> 'Variable':
        """Instantiate `Variable` from an ontology-style data graph.

        Parameters
        ----------
        id : str
            The node identifier for this object.
        data : dict
            Dictionary mapping node ids to their predicate lists.

        Returns
        -------
        Variable
            The populated `Variable` instance.
        """
        self = Variable()
        self.id = id
        mydata = data[id]
        for key in mydata:
            value = mydata[key]
            obj = None
            if key == "type":
                for val in value:
                    self.type = val["@id"]
        
            elif key == "calibratedVia":
                for val in value:
                    if "@id" in val:
                        obj = Calibration.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
            
                    self.calibratedVias.append(obj)
        
            elif key == "foundInDataset":
                for val in value:
                    obj = val["@id"]                        
                    self.foundInDataset = obj
        
            elif key == "foundInTable":
                for val in value:
                    obj = val["@id"]                        
                    self.foundInTable = obj
        
            elif key == "hasArchiveType":
                for val in value:
                    obj = ArchiveType.from_synonym(re.sub("^.*?#", "", val["@id"]))
                                    
                    self.archiveType = obj
        
            elif key == "hasColumnNumber":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.columnNumber = obj
        
            elif key == "hasDescription":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.description = obj
        
            elif key == "hasInstrument":
                for val in value:
                    obj = val["@id"]                        
                    self.instrument = obj
        
            elif key == "hasInterpretation":
                for val in value:
                    if "@id" in val:
                        obj = Interpretation.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
            
                    self.interpretations.append(obj)
        
            elif key == "hasMaxValue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.maxValue = obj
        
            elif key == "hasMeanValue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.meanValue = obj
        
            elif key == "hasMedianValue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.medianValue = obj
        
            elif key == "hasMinValue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.minValue = obj
        
            elif key == "hasMissingValue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.missingValue = obj
        
            elif key == "hasName":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.name = obj
        
            elif key == "hasNotes":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.notes = obj
        
            elif key == "hasPhysicalSample":
                for val in value:
                    if "@id" in val:
                        obj = PhysicalSample.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
            
                    self.physicalSamples.append(obj)
        
            elif key == "hasProxy":
                for val in value:
                    obj = PaleoProxy.from_synonym(re.sub("^.*?#", "", val["@id"]))
                                    
                    self.proxy = obj
        
            elif key == "hasProxyGeneral":
                for val in value:
                    obj = PaleoProxyGeneral.from_synonym(re.sub("^.*?#", "", val["@id"]))
                                    
                    self.proxyGeneral = obj
        
            elif key == "hasResolution":
                for val in value:
                    if "@id" in val:
                        obj = Resolution.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
                                    
                    self.resolution = obj
        
            elif key == "hasStandardVariable":
                for val in value:
                    obj = PaleoVariable.from_synonym(re.sub("^.*?#", "", val["@id"]))
                                    
                    self.standardVariable = obj
        
            elif key == "hasType":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.variableType = obj
        
            elif key == "hasUncertainty":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.uncertainty = obj
        
            elif key == "hasUncertaintyAnalytical":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.uncertaintyAnalytical = obj
        
            elif key == "hasUncertaintyReproducibility":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.uncertaintyReproducibility = obj
        
            elif key == "hasUnits":
                for val in value:
                    obj = PaleoUnit.from_synonym(re.sub("^.*?#", "", val["@id"]))
                                    
                    self.units = obj
        
            elif key == "hasValues":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.values = obj
        
            elif key == "hasVariableId":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.variableId = obj
        
            elif key == "isComposite":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.composite = obj
        
            elif key == "isPrimary":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.primary = obj
        
            elif key == "partOfCompilation":
                for val in value:
                    if "@id" in val:
                        obj = Compilation.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
            
                    self.partOfCompilations.append(obj)
            else:
                for val in value:
                    obj = None
                    if "@id" in val:
                        obj = data[val["@id"]]
                    elif "@value" in val:
                        obj = val["@value"]
                    self.set_non_standard_property(key, obj)
            
        return self

    def to_data(self, data={}):
        """Serialize the object into a JSON-LD compatible dictionary.

        Parameters
        ----------
        data : dict, optional
            Existing data dictionary to extend.

        Returns
        -------
        dict
            The updated data dictionary.
        """
        data[self.id] = {}
        data[self.id]["type"] = [
            {
                "@id": self.type,
                "@type": "uri"
            }
        ]

        if len(self.calibratedVias):
            data[self.id]["calibratedVia"] = []
        for value_obj in self.calibratedVias:
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["calibratedVia"].append(obj)

        if len(self.interpretations):
            data[self.id]["hasInterpretation"] = []
        for value_obj in self.interpretations:
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasInterpretation"].append(obj)

        if len(self.partOfCompilations):
            data[self.id]["partOfCompilation"] = []
        for value_obj in self.partOfCompilations:
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["partOfCompilation"].append(obj)

        if len(self.physicalSamples):
            data[self.id]["hasPhysicalSample"] = []
        for value_obj in self.physicalSamples:
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasPhysicalSample"].append(obj)

        if self.archiveType:
            value_obj = self.archiveType
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasArchiveType"] = [obj]
                

        if self.columnNumber:
            value_obj = self.columnNumber
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#integer"
            }
            data[self.id]["hasColumnNumber"] = [obj]
                

        if self.composite:
            value_obj = self.composite
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#boolean"
            }
            data[self.id]["isComposite"] = [obj]
                

        if self.description:
            value_obj = self.description
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasDescription"] = [obj]
                

        if self.foundInDataset:
            value_obj = self.foundInDataset
            obj = {
                "@id": value_obj,
                "@type": "uri"
            }
            data[self.id]["foundInDataset"] = [obj]
                

        if self.foundInTable:
            value_obj = self.foundInTable
            obj = {
                "@id": value_obj,
                "@type": "uri"
            }
            data[self.id]["foundInTable"] = [obj]
                

        if self.instrument:
            value_obj = self.instrument
            obj = {
                "@id": value_obj,
                "@type": "uri"
            }
            data[self.id]["hasInstrument"] = [obj]
                

        if self.maxValue:
            value_obj = self.maxValue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#float"
            }
            data[self.id]["hasMaxValue"] = [obj]
                

        if self.meanValue:
            value_obj = self.meanValue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#float"
            }
            data[self.id]["hasMeanValue"] = [obj]
                

        if self.medianValue:
            value_obj = self.medianValue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#float"
            }
            data[self.id]["hasMedianValue"] = [obj]
                

        if self.minValue:
            value_obj = self.minValue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#float"
            }
            data[self.id]["hasMinValue"] = [obj]
                

        if self.missingValue:
            value_obj = self.missingValue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasMissingValue"] = [obj]
                

        if self.name:
            value_obj = self.name
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasName"] = [obj]
                

        if self.notes:
            value_obj = self.notes
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasNotes"] = [obj]
                

        if self.primary:
            value_obj = self.primary
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#boolean"
            }
            data[self.id]["isPrimary"] = [obj]
                

        if self.proxy:
            value_obj = self.proxy
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasProxy"] = [obj]
                

        if self.proxyGeneral:
            value_obj = self.proxyGeneral
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasProxyGeneral"] = [obj]
                

        if self.resolution:
            value_obj = self.resolution
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasResolution"] = [obj]
                

        if self.standardVariable:
            value_obj = self.standardVariable
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasStandardVariable"] = [obj]
                

        if self.uncertainty:
            value_obj = self.uncertainty
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasUncertainty"] = [obj]
                

        if self.uncertaintyAnalytical:
            value_obj = self.uncertaintyAnalytical
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasUncertaintyAnalytical"] = [obj]
                

        if self.uncertaintyReproducibility:
            value_obj = self.uncertaintyReproducibility
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasUncertaintyReproducibility"] = [obj]
                

        if self.units:
            value_obj = self.units
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasUnits"] = [obj]
                

        if self.values:
            value_obj = self.values
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasValues"] = [obj]
                

        if self.variableId:
            value_obj = self.variableId
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasVariableId"] = [obj]
                

        if self.variableType:
            value_obj = self.variableType
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasType"] = [obj]
                
        for key in self.misc:
            value = self.misc[key]
            data[self.id][key] = []
            ptype = None
            tp = type(value).__name__
            if tp == "int":
                ptype = "http://www.w3.org/2001/XMLSchema#integer"
            elif tp == "float" or tp == "double":
                ptype = "http://www.w3.org/2001/XMLSchema#float"
            elif tp == "str":
                if re.match(r"\d{4}-\d{2}-\d{2}( |T)\d{2}:\d{2}:\d{2}", value):
                    ptype = "http://www.w3.org/2001/XMLSchema#datetime"   
                elif re.match(r"\d{4}-\d{2}-\d{2}", value):
                    ptype = "http://www.w3.org/2001/XMLSchema#date"
                else:
                    ptype = "http://www.w3.org/2001/XMLSchema#string"
            elif tp == "bool":
                ptype = "http://www.w3.org/2001/XMLSchema#boolean"

            data[self.id][key].append({
                "@value": value,
                "@type": "literal",
                "@datatype": ptype
            })
        
        return data

    def to_json(self):
        """Return a lightweight JSON representation (used by LiPD).

        Returns
        -------
        dict
            A dictionary representation of this object.
        """
        data = {
            "@id": self.id
        }

        if len(self.calibratedVias):
            data["calibration"] = []
        for value_obj in self.calibratedVias:
            obj = value_obj.to_json()
            data["calibration"].append(obj)

        if len(self.interpretations):
            data["interpretation"] = []
        for value_obj in self.interpretations:
            obj = value_obj.to_json()
            data["interpretation"].append(obj)

        if len(self.partOfCompilations):
            data["inCompilationBeta"] = []
        for value_obj in self.partOfCompilations:
            obj = value_obj.to_json()
            data["inCompilationBeta"].append(obj)

        if len(self.physicalSamples):
            data["physicalSample"] = []
        for value_obj in self.physicalSamples:
            obj = value_obj.to_json()
            data["physicalSample"].append(obj)

        if self.archiveType:
            value_obj = self.archiveType
            obj = value_obj.to_json()
            data["archiveType"] = obj

        if self.columnNumber:
            value_obj = self.columnNumber
            obj = value_obj
            data["number"] = obj

        if self.composite:
            value_obj = self.composite
            obj = value_obj
            data["isComposite"] = obj

        if self.description:
            value_obj = self.description
            obj = value_obj
            data["description"] = obj

        if self.instrument:
            value_obj = self.instrument
            obj = value_obj
            data["measurementInstrument"] = obj

        if self.maxValue:
            value_obj = self.maxValue
            obj = value_obj
            data["hasMaxValue"] = obj

        if self.meanValue:
            value_obj = self.meanValue
            obj = value_obj
            data["hasMeanValue"] = obj

        if self.medianValue:
            value_obj = self.medianValue
            obj = value_obj
            data["hasMedianValue"] = obj

        if self.minValue:
            value_obj = self.minValue
            obj = value_obj
            data["hasMinValue"] = obj

        if self.missingValue:
            value_obj = self.missingValue
            obj = value_obj
            data["missingValue"] = obj

        if self.name:
            value_obj = self.name
            obj = value_obj
            data["variableName"] = obj

        if self.notes:
            value_obj = self.notes
            obj = value_obj
            data["notes"] = obj

        if self.primary:
            value_obj = self.primary
            obj = value_obj
            data["isPrimary"] = obj

        if self.proxy:
            value_obj = self.proxy
            obj = value_obj.to_json()
            data["proxy"] = obj

        if self.proxyGeneral:
            value_obj = self.proxyGeneral
            obj = value_obj.to_json()
            data["proxyGeneral"] = obj

        if self.resolution:
            value_obj = self.resolution
            obj = value_obj.to_json()
            data["resolution"] = obj

        if self.standardVariable:
            value_obj = self.standardVariable
            obj = value_obj.to_json()
            data["hasStandardVariable"] = obj

        if self.uncertainty:
            value_obj = self.uncertainty
            obj = value_obj
            data["uncertainty"] = obj

        if self.uncertaintyAnalytical:
            value_obj = self.uncertaintyAnalytical
            obj = value_obj
            data["uncertaintyAnalytical"] = obj

        if self.uncertaintyReproducibility:
            value_obj = self.uncertaintyReproducibility
            obj = value_obj
            data["uncertaintyReproducibility"] = obj

        if self.units:
            value_obj = self.units
            obj = value_obj.to_json()
            data["units"] = obj

        if self.values:
            value_obj = self.values
            obj = value_obj
            data["hasValues"] = obj

        if self.variableId:
            value_obj = self.variableId
            obj = value_obj
            data["TSid"] = obj

        if self.variableType:
            value_obj = self.variableType
            obj = value_obj
            data["variableType"] = obj

        for key in self.misc:
            value = self.misc[key]
            data[key] = value
                   
        return data

    @staticmethod
    def from_json(data) -> 'Variable':
        """Instantiate `Variable` from its lightweight JSON representation.

        Parameters
        ----------
        data : dict
            The JSON dictionary to parse.

        Returns
        -------
        Variable
            The populated `Variable` instance.
        """
        self = Variable()
        for key in data:
            pvalue = data[key]
            if key == "@id":
                self.id = pvalue
            elif key == "TSid":
                    value = pvalue
                    obj = value
                    self.variableId = obj
            elif key == "archiveType":
                    value = pvalue
                    obj = ArchiveType.from_synonym(re.sub("^.*?#", "", value))
                    self.archiveType = obj
            elif key == "calibration":
                for value in pvalue:
                    obj = Calibration.from_json(value)
                    self.calibratedVias.append(obj)
            elif key == "description":
                    value = pvalue
                    obj = value
                    self.description = obj
            elif key == "foundInDataset":
                    value = pvalue
                    obj = value
                    self.foundInDataset = obj
            elif key == "foundInTable":
                    value = pvalue
                    obj = value
                    self.foundInTable = obj
            elif key == "hasMaxValue":
                    value = pvalue
                    obj = value
                    self.maxValue = obj
            elif key == "hasMeanValue":
                    value = pvalue
                    obj = value
                    self.meanValue = obj
            elif key == "hasMedianValue":
                    value = pvalue
                    obj = value
                    self.medianValue = obj
            elif key == "hasMinValue":
                    value = pvalue
                    obj = value
                    self.minValue = obj
            elif key == "hasStandardVariable":
                    value = pvalue
                    obj = PaleoVariable.from_synonym(re.sub("^.*?#", "", value))
                    self.standardVariable = obj
            elif key == "hasValues":
                    value = pvalue
                    obj = value
                    self.values = obj
            elif key == "inCompilationBeta":
                for value in pvalue:
                    obj = Compilation.from_json(value)
                    self.partOfCompilations.append(obj)
            elif key == "interpretation":
                for value in pvalue:
                    obj = Interpretation.from_json(value)
                    self.interpretations.append(obj)
            elif key == "isComposite":
                    value = pvalue
                    obj = value
                    self.composite = obj
            elif key == "isPrimary":
                    value = pvalue
                    obj = value
                    self.primary = obj
            elif key == "measurementInstrument":
                    value = pvalue
                    obj = value
                    self.instrument = obj
            elif key == "missingValue":
                    value = pvalue
                    obj = value
                    self.missingValue = obj
            elif key == "notes":
                    value = pvalue
                    obj = value
                    self.notes = obj
            elif key == "number":
                    value = pvalue
                    obj = value
                    self.columnNumber = obj
            elif key == "physicalSample":
                for value in pvalue:
                    obj = PhysicalSample.from_json(value)
                    self.physicalSamples.append(obj)
            elif key == "proxy":
                    value = pvalue
                    obj = PaleoProxy.from_synonym(re.sub("^.*?#", "", value))
                    self.proxy = obj
            elif key == "proxyGeneral":
                    value = pvalue
                    obj = PaleoProxyGeneral.from_synonym(re.sub("^.*?#", "", value))
                    self.proxyGeneral = obj
            elif key == "resolution":
                    value = pvalue
                    obj = Resolution.from_json(value)
                    self.resolution = obj
            elif key == "uncertainty":
                    value = pvalue
                    obj = value
                    self.uncertainty = obj
            elif key == "uncertaintyAnalytical":
                    value = pvalue
                    obj = value
                    self.uncertaintyAnalytical = obj
            elif key == "uncertaintyReproducibility":
                    value = pvalue
                    obj = value
                    self.uncertaintyReproducibility = obj
            elif key == "units":
                    value = pvalue
                    obj = PaleoUnit.from_synonym(re.sub("^.*?#", "", value))
                    self.units = obj
            elif key == "variableName":
                    value = pvalue
                    obj = value
                    self.name = obj
            elif key == "variableType":
                    value = pvalue
                    obj = value
                    self.variableType = obj
            else:
                self.set_non_standard_property(key, pvalue)
                   
        return self

    def set_non_standard_property(self, key, value):
        """Store a predicate that is not defined in the ontology schema.

        This is useful for forward-compatibility with new properties that are
        not yet part of the official schema.

        Parameters
        ----------
        key : str
            The property name.
        value : any
            The property value.
        """
        if key not in self.misc:
            self.misc[key] = value
    
    def get_non_standard_property(self, key):
        """Return a single non-standard property by key.

        Parameters
        ----------
        key : str
            The property name.

        Returns
        -------
        any
            The property value.
        """
        return self.misc[key]
                
    def get_all_non_standard_properties(self):
        """Return the dictionary of all non-standard properties.

        Returns
        -------
        dict
            Dictionary of all non-standard properties.
        """
        return self.misc

    def add_non_standard_property(self, key, value):
        """Append a value to a list-valued non-standard property.

        Parameters
        ----------
        key : str
            The property name.
        value : any
            The value to append.
        """
        if key not in self.misc:
            self.misc[key] = []
        self.misc[key].append(value)
        
    def getArchiveType(self) -> ArchiveType:
        """Get archiveType.

        Returns
        -------
        ArchiveType
            The current value of archiveType.
        """
        return self.archiveType

    def setArchiveType(self, archiveType:ArchiveType):
        """Set archiveType.

        Parameters
        ----------
        archiveType : ArchiveType
            The value to assign.
        """
        assert isinstance(archiveType, ArchiveType), f"Error: '{archiveType}' is not of type ArchiveType\nYou can create a new ArchiveType object from a string using the following syntax:\n- Fetch existing ArchiveType by synonym: ArchiveType.from_synonym(\"{archiveType}\")\n- Create a new custom ArchiveType: ArchiveType(\"{archiveType}\")"
        self.archiveType = archiveType
    
    def getCalibratedVias(self) -> list[Calibration]:
        """Get calibratedVias list.

        Returns
        -------
        list[Calibration]
            A list of Calibration objects.
        """
        return self.calibratedVias

    def setCalibratedVias(self, calibratedVias:list[Calibration]):
        """Set the calibratedVias list.

        Parameters
        ----------
        calibratedVias : list[Calibration]
            The list to assign.
        """
        assert isinstance(calibratedVias, list), "Error: calibratedVias is not a list"
        assert all(isinstance(x, Calibration) for x in calibratedVias), f"Error: '{calibratedVias}' is not of type Calibration"
        self.calibratedVias = calibratedVias

    def addCalibratedVia(self, calibratedVias:Calibration):
        """Add a value to the calibratedVias list.

        Parameters
        ----------
        calibratedVias : Calibration
            The value to append.
        """
        assert isinstance(calibratedVias, Calibration), f"Error: '{calibratedVias}' is not of type Calibration"
        self.calibratedVias.append(calibratedVias)
        
    def getColumnNumber(self) -> int:
        """Get columnNumber.

        Returns
        -------
        int
            The current value of columnNumber.
        """
        return self.columnNumber

    def setColumnNumber(self, columnNumber:int):
        """Set columnNumber.

        Parameters
        ----------
        columnNumber : int
            The value to assign.
        """
        assert isinstance(columnNumber, int), f"Error: '{columnNumber}' is not of type int"
        self.columnNumber = columnNumber
    
    def getDescription(self) -> str:
        """Get description.

        Returns
        -------
        str
            The current value of description.
        """
        return self.description

    def setDescription(self, description:str):
        """Set description.

        Parameters
        ----------
        description : str
            The value to assign.
        """
        assert isinstance(description, str), f"Error: '{description}' is not of type str"
        self.description = description
    
    def getFoundInDataset(self) -> object:
        """Get foundInDataset.

        Returns
        -------
        object
            The current value of foundInDataset.
        """
        return self.foundInDataset

    def setFoundInDataset(self, foundInDataset:object):
        """Set foundInDataset.

        Parameters
        ----------
        foundInDataset : object
            The value to assign.
        """
        assert isinstance(foundInDataset, object), f"Error: '{foundInDataset}' is not of type object"
        self.foundInDataset = foundInDataset
    
    def getFoundInTable(self) -> object:
        """Get foundInTable.

        Returns
        -------
        object
            The current value of foundInTable.
        """
        return self.foundInTable

    def setFoundInTable(self, foundInTable:object):
        """Set foundInTable.

        Parameters
        ----------
        foundInTable : object
            The value to assign.
        """
        assert isinstance(foundInTable, object), f"Error: '{foundInTable}' is not of type object"
        self.foundInTable = foundInTable
    
    def getInstrument(self) -> object:
        """Get instrument.

        Returns
        -------
        object
            The current value of instrument.
        """
        return self.instrument

    def setInstrument(self, instrument:object):
        """Set instrument.

        Parameters
        ----------
        instrument : object
            The value to assign.
        """
        assert isinstance(instrument, object), f"Error: '{instrument}' is not of type object"
        self.instrument = instrument
    
    def getInterpretations(self) -> list[Interpretation]:
        """Get interpretations list.

        Returns
        -------
        list[Interpretation]
            A list of Interpretation objects.
        """
        return self.interpretations

    def setInterpretations(self, interpretations:list[Interpretation]):
        """Set the interpretations list.

        Parameters
        ----------
        interpretations : list[Interpretation]
            The list to assign.
        """
        assert isinstance(interpretations, list), "Error: interpretations is not a list"
        assert all(isinstance(x, Interpretation) for x in interpretations), f"Error: '{interpretations}' is not of type Interpretation"
        self.interpretations = interpretations

    def addInterpretation(self, interpretations:Interpretation):
        """Add a value to the interpretations list.

        Parameters
        ----------
        interpretations : Interpretation
            The value to append.
        """
        assert isinstance(interpretations, Interpretation), f"Error: '{interpretations}' is not of type Interpretation"
        self.interpretations.append(interpretations)
        
    def getMaxValue(self) -> float:
        """Get maxValue.

        Returns
        -------
        float
            The current value of maxValue.
        """
        return self.maxValue

    def setMaxValue(self, maxValue:float):
        """Set maxValue.

        Parameters
        ----------
        maxValue : float
            The value to assign.
        """
        assert isinstance(maxValue, float), f"Error: '{maxValue}' is not of type float"
        self.maxValue = maxValue
    
    def getMeanValue(self) -> float:
        """Get meanValue.

        Returns
        -------
        float
            The current value of meanValue.
        """
        return self.meanValue

    def setMeanValue(self, meanValue:float):
        """Set meanValue.

        Parameters
        ----------
        meanValue : float
            The value to assign.
        """
        assert isinstance(meanValue, float), f"Error: '{meanValue}' is not of type float"
        self.meanValue = meanValue
    
    def getMedianValue(self) -> float:
        """Get medianValue.

        Returns
        -------
        float
            The current value of medianValue.
        """
        return self.medianValue

    def setMedianValue(self, medianValue:float):
        """Set medianValue.

        Parameters
        ----------
        medianValue : float
            The value to assign.
        """
        assert isinstance(medianValue, float), f"Error: '{medianValue}' is not of type float"
        self.medianValue = medianValue
    
    def getMinValue(self) -> float:
        """Get minValue.

        Returns
        -------
        float
            The current value of minValue.
        """
        return self.minValue

    def setMinValue(self, minValue:float):
        """Set minValue.

        Parameters
        ----------
        minValue : float
            The value to assign.
        """
        assert isinstance(minValue, float), f"Error: '{minValue}' is not of type float"
        self.minValue = minValue
    
    def getMissingValue(self) -> str:
        """Get missingValue.

        Returns
        -------
        str
            The current value of missingValue.
        """
        return self.missingValue

    def setMissingValue(self, missingValue:str):
        """Set missingValue.

        Parameters
        ----------
        missingValue : str
            The value to assign.
        """
        assert isinstance(missingValue, str), f"Error: '{missingValue}' is not of type str"
        self.missingValue = missingValue
    
    def getName(self) -> str:
        """Get name.

        Returns
        -------
        str
            The current value of name.
        """
        return self.name

    def setName(self, name:str):
        """Set name.

        Parameters
        ----------
        name : str
            The value to assign.
        """
        assert isinstance(name, str), f"Error: '{name}' is not of type str"
        self.name = name
    
    def getNotes(self) -> str:
        """Get notes.

        Returns
        -------
        str
            The current value of notes.
        """
        return self.notes

    def setNotes(self, notes:str):
        """Set notes.

        Parameters
        ----------
        notes : str
            The value to assign.
        """
        assert isinstance(notes, str), f"Error: '{notes}' is not of type str"
        self.notes = notes
    
    def getPartOfCompilations(self) -> list[Compilation]:
        """Get partOfCompilations list.

        Returns
        -------
        list[Compilation]
            A list of Compilation objects.
        """
        return self.partOfCompilations

    def setPartOfCompilations(self, partOfCompilations:list[Compilation]):
        """Set the partOfCompilations list.

        Parameters
        ----------
        partOfCompilations : list[Compilation]
            The list to assign.
        """
        assert isinstance(partOfCompilations, list), "Error: partOfCompilations is not a list"
        assert all(isinstance(x, Compilation) for x in partOfCompilations), f"Error: '{partOfCompilations}' is not of type Compilation"
        self.partOfCompilations = partOfCompilations

    def addPartOfCompilation(self, partOfCompilations:Compilation):
        """Add a value to the partOfCompilations list.

        Parameters
        ----------
        partOfCompilations : Compilation
            The value to append.
        """
        assert isinstance(partOfCompilations, Compilation), f"Error: '{partOfCompilations}' is not of type Compilation"
        self.partOfCompilations.append(partOfCompilations)
        
    def getPhysicalSamples(self) -> list[PhysicalSample]:
        """Get physicalSamples list.

        Returns
        -------
        list[PhysicalSample]
            A list of PhysicalSample objects.
        """
        return self.physicalSamples

    def setPhysicalSamples(self, physicalSamples:list[PhysicalSample]):
        """Set the physicalSamples list.

        Parameters
        ----------
        physicalSamples : list[PhysicalSample]
            The list to assign.
        """
        assert isinstance(physicalSamples, list), "Error: physicalSamples is not a list"
        assert all(isinstance(x, PhysicalSample) for x in physicalSamples), f"Error: '{physicalSamples}' is not of type PhysicalSample"
        self.physicalSamples = physicalSamples

    def addPhysicalSample(self, physicalSamples:PhysicalSample):
        """Add a value to the physicalSamples list.

        Parameters
        ----------
        physicalSamples : PhysicalSample
            The value to append.
        """
        assert isinstance(physicalSamples, PhysicalSample), f"Error: '{physicalSamples}' is not of type PhysicalSample"
        self.physicalSamples.append(physicalSamples)
        
    def getProxy(self) -> PaleoProxy:
        """Get proxy.

        Returns
        -------
        PaleoProxy
            The current value of proxy.
        """
        return self.proxy

    def setProxy(self, proxy:PaleoProxy):
        """Set proxy.

        Parameters
        ----------
        proxy : PaleoProxy
            The value to assign.
        """
        assert isinstance(proxy, PaleoProxy), f"Error: '{proxy}' is not of type PaleoProxy\nYou can create a new PaleoProxy object from a string using the following syntax:\n- Fetch existing PaleoProxy by synonym: PaleoProxy.from_synonym(\"{proxy}\")\n- Create a new custom PaleoProxy: PaleoProxy(\"{proxy}\")"
        self.proxy = proxy
    
    def getProxyGeneral(self) -> PaleoProxyGeneral:
        """Get proxyGeneral.

        Returns
        -------
        PaleoProxyGeneral
            The current value of proxyGeneral.
        """
        return self.proxyGeneral

    def setProxyGeneral(self, proxyGeneral:PaleoProxyGeneral):
        """Set proxyGeneral.

        Parameters
        ----------
        proxyGeneral : PaleoProxyGeneral
            The value to assign.
        """
        assert isinstance(proxyGeneral, PaleoProxyGeneral), f"Error: '{proxyGeneral}' is not of type PaleoProxyGeneral\nYou can create a new PaleoProxyGeneral object from a string using the following syntax:\n- Fetch existing PaleoProxyGeneral by synonym: PaleoProxyGeneral.from_synonym(\"{proxyGeneral}\")\n- Create a new custom PaleoProxyGeneral: PaleoProxyGeneral(\"{proxyGeneral}\")"
        self.proxyGeneral = proxyGeneral
    
    def getResolution(self) -> Resolution:
        """Get resolution.

        Returns
        -------
        Resolution
            The current value of resolution.
        """
        return self.resolution

    def setResolution(self, resolution:Resolution):
        """Set resolution.

        Parameters
        ----------
        resolution : Resolution
            The value to assign.
        """
        assert isinstance(resolution, Resolution), f"Error: '{resolution}' is not of type Resolution"
        self.resolution = resolution
    
    def getStandardVariable(self) -> PaleoVariable:
        """Get standardVariable.

        Returns
        -------
        PaleoVariable
            The current value of standardVariable.
        """
        return self.standardVariable

    def setStandardVariable(self, standardVariable:PaleoVariable):
        """Set standardVariable.

        Parameters
        ----------
        standardVariable : PaleoVariable
            The value to assign.
        """
        assert isinstance(standardVariable, PaleoVariable), f"Error: '{standardVariable}' is not of type PaleoVariable\nYou can create a new PaleoVariable object from a string using the following syntax:\n- Fetch existing PaleoVariable by synonym: PaleoVariable.from_synonym(\"{standardVariable}\")\n- Create a new custom PaleoVariable: PaleoVariable(\"{standardVariable}\")"
        self.standardVariable = standardVariable
    
    def getUncertainty(self) -> str:
        """Get uncertainty.

        Returns
        -------
        str
            The current value of uncertainty.
        """
        return self.uncertainty

    def setUncertainty(self, uncertainty:str):
        """Set uncertainty.

        Parameters
        ----------
        uncertainty : str
            The value to assign.
        """
        assert isinstance(uncertainty, str), f"Error: '{uncertainty}' is not of type str"
        self.uncertainty = uncertainty
    
    def getUncertaintyAnalytical(self) -> str:
        """Get uncertaintyAnalytical.

        Returns
        -------
        str
            The current value of uncertaintyAnalytical.
        """
        return self.uncertaintyAnalytical

    def setUncertaintyAnalytical(self, uncertaintyAnalytical:str):
        """Set uncertaintyAnalytical.

        Parameters
        ----------
        uncertaintyAnalytical : str
            The value to assign.
        """
        assert isinstance(uncertaintyAnalytical, str), f"Error: '{uncertaintyAnalytical}' is not of type str"
        self.uncertaintyAnalytical = uncertaintyAnalytical
    
    def getUncertaintyReproducibility(self) -> str:
        """Get uncertaintyReproducibility.

        Returns
        -------
        str
            The current value of uncertaintyReproducibility.
        """
        return self.uncertaintyReproducibility

    def setUncertaintyReproducibility(self, uncertaintyReproducibility:str):
        """Set uncertaintyReproducibility.

        Parameters
        ----------
        uncertaintyReproducibility : str
            The value to assign.
        """
        assert isinstance(uncertaintyReproducibility, str), f"Error: '{uncertaintyReproducibility}' is not of type str"
        self.uncertaintyReproducibility = uncertaintyReproducibility
    
    def getUnits(self) -> PaleoUnit:
        """Get units.

        Returns
        -------
        PaleoUnit
            The current value of units.
        """
        return self.units

    def setUnits(self, units:PaleoUnit):
        """Set units.

        Parameters
        ----------
        units : PaleoUnit
            The value to assign.
        """
        assert isinstance(units, PaleoUnit), f"Error: '{units}' is not of type PaleoUnit\nYou can create a new PaleoUnit object from a string using the following syntax:\n- Fetch existing PaleoUnit by synonym: PaleoUnit.from_synonym(\"{units}\")\n- Create a new custom PaleoUnit: PaleoUnit(\"{units}\")"
        self.units = units
    
    def getValues(self) -> str:
        """Get values.

        Returns
        -------
        str
            The current value of values.
        """
        return self.values

    def setValues(self, values:str):
        """Set values.

        Parameters
        ----------
        values : str
            The value to assign.
        """
        assert isinstance(values, str), f"Error: '{values}' is not of type str"
        self.values = values
    
    def getVariableId(self) -> str:
        """Get variableId.

        Returns
        -------
        str
            The current value of variableId.
        """
        return self.variableId

    def setVariableId(self, variableId:str):
        """Set variableId.

        Parameters
        ----------
        variableId : str
            The value to assign.
        """
        assert isinstance(variableId, str), f"Error: '{variableId}' is not of type str"
        self.variableId = variableId
    
    def getVariableType(self) -> str:
        """Get variableType.

        Returns
        -------
        str
            The current value of variableType.
        """
        return self.variableType

    def setVariableType(self, variableType:str):
        """Set variableType.

        Parameters
        ----------
        variableType : str
            The value to assign.
        """
        assert isinstance(variableType, str), f"Error: '{variableType}' is not of type str"
        self.variableType = variableType
    
    def isComposite(self) -> bool:
        """Get composite.

        Returns
        -------
        bool
            The current value of composite.
        """
        return self.composite

    def setComposite(self, composite:bool):
        """Set composite.

        Parameters
        ----------
        composite : bool
            The value to assign.
        """
        assert isinstance(composite, bool), f"Error: '{composite}' is not of type bool"
        self.composite = composite
    
    def isPrimary(self) -> bool:
        """Get primary.

        Returns
        -------
        bool
            The current value of primary.
        """
        return self.primary

    def setPrimary(self, primary:bool):
        """Set primary.

        Parameters
        ----------
        primary : bool
            The value to assign.
        """
        assert isinstance(primary, bool), f"Error: '{primary}' is not of type bool"
        self.primary = primary
    


import re

"""
GLOBAL LIST OF REGEXES
"""

# re_clean_url = re.compile(r'\"\<\>\#\%\{\}\|\^\~\'\[\]\`')

# GENERAL
re_model_name = re.compile(r'(paleo|chron)[a-zA-Z]*(\d+)(model)?[a-zA-Z]*(\d)', re.I)
re_table_name = re.compile(r'(paleo|chron)(\d+)(model|measurement)(\d+)(ensemble|summary|distribution)?(\d+)?')
re_sci_notation = re.compile(r"(\d+)([.]\d+)(e-\d+)")
re_url = re.compile(r"^(http:\/\/www\.|https:\/\/www\.|http:\/\/|https:\/\/)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$")
re_ensemble_collapse = re.compile(r'([\w]*)\-([\w]*)\-ens')

# EXCEL
re_var_w_units = re.compile(r'([\w\s\.\-\_\:\/\\\"\'\`\#\+\{\}\[\]\;\*]+)\(?([\w\s\.\-\_\:\/\\\"\'\`\#\+\{\}\[\]\;\*]+)?\)?', re.I)
# todo write a regex for legacy sheet names
re_sheet = re.compile(r'(paleo|chron)[a-zA-Z]*(\d+)(model)?[a-zA-Z]*(\d+)?(measurement|ensemble|summary|distribution)[a-zA-Z]*(\d+)?', re.I)
re_sheet_w_number = re.compile(r"(paleo\d+|chron\d+)(model\d+)?(measurement\d+|ensemble\d+|summary\d+|distribution\d+)?", re.I)
re_table = re.compile(r"(paleo|model|chron|ensemble|distribution|summary)(\d+)")
re_calibration = re.compile(r"calibration_(\w+)")
re_physical = re.compile(r"physicalsample_(\w+)")
re_interpretation = re.compile(r"interpretation(\d)_(\w+)")

# DOI
re_doi = re.compile(r'\b(10[.][0-9]{3,}(?:[.][0-9]+)*\/(?:(?!["&\'<>,])\S)+)\b')

# NOAA
# Convert camelCase to underscore
re_tab_split = re.compile(r'\t+')
first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_cap_re = re.compile('([a-z0-9])([A-Z])')
re_var = re.compile(r'#{2}\s*(\S+)(?:\s+)([\w\W\s]+)')
re_var_split = re.compile(r'(\S+)(?:\s+)([\w\W\s]+)')
re_name_unit = re.compile(r'([\-]?\d+)?\s*[(]?\s*(\w+)?\s*[)]?')
re_name_unit_range = re.compile(r'([\-]?\d+)[\s\(]*(?!to)(\w+)*')
re_chron_var_header = re.compile(r"(\w+\S+)[\s]{0,2}((?:\()?(\w+)?(?:\)))?")
# re_chron_var_desc = re.compile(r"(\w+\S*)\s+(.*)")

# TIME SERIES
re_misc_fetch = re.compile(r'(geo_(\w+)|climateInterpretation_(\w+)|calibration_(\w+)|paleoData_(\w+))')
re_pub_fetch = re.compile(r'pub1_(citation|year|DOI|author|publisher|title|type|volume|issue|journal|link|pubDataUrl|abstract|pages)')

re_pub_valid = re.compile(r'pub(\d)_(citation|year|DOI|author|publisher|title|type|volume|issue|journal|link|pubDataUrl|abstract|pages)', re.I)
re_fund_valid = re.compile(r'funding(\d)_(grant|agency)', re.I)

re_pub_invalid = re.compile(r'pub_(\w+)|pub(\d)_(\w+)|pub(\d)(\w+)|pub(\w+)')
re_fund_invalid = re.compile(r'agency|grant|funding_agency|funding_grant')
re_geo_invalid = re.compile(r'geo(\w+)|geo_(\w+)')
re_paleo_invalid = re.compile(r'paleodata(\w+)|paleodata_(\w+)|measurement(\w+)|measurement_(\w+)')
re_calib_invalid = re.compile(r'calibration(\w+)|calibration_(\w+)')
re_clim_invalid = re.compile(r'climateinterpretation(\w+)|climateinterpretation_(\w+)')

re_pub_nh = re.compile(r'pub(\d)_(\w+)')
re_pub_cc = re.compile(r'pub(\w+)')
re_pub_h = re.compile(r'pub_(\w+)')
re_pub_n = re.compile(r'pub(\d)(\w+)')

# START
re_filter_expr = re.compile(r"((\w+_?)\s*(is|in|greater than|equals|equal|less than|<=|==|>=|=|>|<){1}(.*))")

# PANDAS
re_pandas_x_num = re.compile(r"(year\d?|age\d?|depth\d?)\b")
re_pandas_x_und = re.compile(r"(year|age|depth|yr){1}[_]{1}.*")



##############################
# Auto-generated. Do not Edit
##############################
from pylipd.globals.synonyms import SYNONYMS

class PaleoUnit:
    """Controlled-vocabulary class for `PaleoUnit` terms."""
    synonyms = SYNONYMS["UNITS"]["PaleoUnit"]

    def __init__(self, id, label):
        """Initialize a PaleoUnit term.

        Parameters
        ----------
        id : str
            The full URI identifier for this term.
        label : str
            The human-readable label for this term.
        """
        self.id = id
        self.label = label
    
    def __eq__(self, value: object) -> bool:
        """Check equality based on term ID.

        Parameters
        ----------
        value : object
            The object to compare against.

        Returns
        -------
        bool
            True if the IDs match, False otherwise.
        """
        return self.id == value.id
        
    def getLabel(self):
        """Return the human-readable label of the term.

        Returns
        -------
        str
            The label for this term.
        """
        return self.label

    def getId(self):
        """Return the full URI identifier of the term.

        Returns
        -------
        str
            The URI identifier for this term.
        """
        return self.id
    
    def to_data(self, data={}):
        """Serialize this term to JSON-LD format.

        Parameters
        ----------
        data : dict, optional
            Existing data dictionary to extend.

        Returns
        -------
        dict
            The updated data dictionary.
        """
        data[self.id] ={
            "label": [
                {
                    "@datatype": None,
                    "@type": "literal",
                    "@value": self.label
                }
            ]
        }
        return data

    def to_json(self):
        """Return the plain-text label (used in lightweight JSON).

        Returns
        -------
        str
            The label for this term.
        """
        data = self.label
        return data

    @classmethod
    def from_synonym(cls, synonym):
        """Create a PaleoUnit instance from a synonym string.

        Parameters
        ----------
        synonym : str
            A synonym or alternative name for the term.

        Returns
        -------
        PaleoUnit or None
            The PaleoUnit instance if found, None otherwise.
        """
        if synonym.lower() in PaleoUnit.synonyms:
            synobj = PaleoUnit.synonyms[synonym.lower()]
            return PaleoUnit(synobj['id'], synobj['label'])
        return None
        
class PaleoUnitConstants:
    atomic_ratio = PaleoUnit("http://linked.earth/ontology/paleo_units#atomic_ratio", "atomic ratio")
    cgs = PaleoUnit("http://linked.earth/ontology/paleo_units#cgs", "cgs")
    cm = PaleoUnit("http://linked.earth/ontology/paleo_units#cm", "cm")
    cm_kyr = PaleoUnit("http://linked.earth/ontology/paleo_units#cm_kyr", "cm/kyr")
    cm_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#cm_yr", "cm/yr")
    cm3 = PaleoUnit("http://linked.earth/ontology/paleo_units#cm3", "cm3")
    count = PaleoUnit("http://linked.earth/ontology/paleo_units#count", "count")
    count_century = PaleoUnit("http://linked.earth/ontology/paleo_units#count_century", "count/century")
    count_cm2 = PaleoUnit("http://linked.earth/ontology/paleo_units#count_cm2", "count/cm2")
    count_cm2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#count_cm2_yr", "count/cm2/yr")
    count_cm3 = PaleoUnit("http://linked.earth/ontology/paleo_units#count_cm3", "count/cm3")
    count_g = PaleoUnit("http://linked.earth/ontology/paleo_units#count_g", "count/g")
    count_kyr = PaleoUnit("http://linked.earth/ontology/paleo_units#count_kyr", "count/kyr")
    count_mL = PaleoUnit("http://linked.earth/ontology/paleo_units#count_mL", "count/mL")
    count_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#count_yr", "count/yr")
    cps = PaleoUnit("http://linked.earth/ontology/paleo_units#cps", "cps")
    day = PaleoUnit("http://linked.earth/ontology/paleo_units#day", "day")
    degC = PaleoUnit("http://linked.earth/ontology/paleo_units#degC", "degC")
    degree = PaleoUnit("http://linked.earth/ontology/paleo_units#degree", "degree")
    fraction = PaleoUnit("http://linked.earth/ontology/paleo_units#fraction", "fraction")
    g = PaleoUnit("http://linked.earth/ontology/paleo_units#g", "g")
    g_cm_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#g_cm_yr", "g/cm/yr")
    g_cm2 = PaleoUnit("http://linked.earth/ontology/paleo_units#g_cm2", "g/cm2")
    g_cm2_kyr = PaleoUnit("http://linked.earth/ontology/paleo_units#g_cm2_kyr", "g/cm2/kyr")
    g_cm2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#g_cm2_yr", "g/cm2/yr")
    g_cm3 = PaleoUnit("http://linked.earth/ontology/paleo_units#g_cm3", "g/cm3")
    g_L = PaleoUnit("http://linked.earth/ontology/paleo_units#g_L", "g/L")
    g_m2 = PaleoUnit("http://linked.earth/ontology/paleo_units#g_m2", "g/m2")
    g_m2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#g_m2_yr", "g/m2/yr")
    grayscale = PaleoUnit("http://linked.earth/ontology/paleo_units#grayscale", "grayscale")
    kg_m2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#kg_m2_yr", "kg/m2/yr")
    kg_m3 = PaleoUnit("http://linked.earth/ontology/paleo_units#kg_m3", "kg/m3")
    km2 = PaleoUnit("http://linked.earth/ontology/paleo_units#km2", "km2")
    km3 = PaleoUnit("http://linked.earth/ontology/paleo_units#km3", "km3")
    log_mg_L_ = PaleoUnit("http://linked.earth/ontology/paleo_units#log_mg_L_", "log(mg/L)")
    m = PaleoUnit("http://linked.earth/ontology/paleo_units#m", "m")
    m3_kg = PaleoUnit("http://linked.earth/ontology/paleo_units#m3_kg", "m3/kg")
    mg = PaleoUnit("http://linked.earth/ontology/paleo_units#mg", "mg")
    mg_cm2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#mg_cm2_yr", "mg/cm2/yr")
    mg_g = PaleoUnit("http://linked.earth/ontology/paleo_units#mg_g", "mg/g")
    mg_kg = PaleoUnit("http://linked.earth/ontology/paleo_units#mg_kg", "mg/kg")
    mg_L = PaleoUnit("http://linked.earth/ontology/paleo_units#mg_L", "mg/L")
    mm = PaleoUnit("http://linked.earth/ontology/paleo_units#mm", "mm")
    mm_day = PaleoUnit("http://linked.earth/ontology/paleo_units#mm_day", "mm/day")
    mm_season = PaleoUnit("http://linked.earth/ontology/paleo_units#mm_season", "mm/season")
    mm_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#mm_yr", "mm/yr")
    mmol_mol = PaleoUnit("http://linked.earth/ontology/paleo_units#mmol_mol", "mmol/mol")
    months_year = PaleoUnit("http://linked.earth/ontology/paleo_units#months_year", "months/year")
    needsToBeChanged = PaleoUnit("http://linked.earth/ontology/paleo_units#needsToBeChanged", "needsToBeChanged")
    ng = PaleoUnit("http://linked.earth/ontology/paleo_units#ng", "ng")
    ng_g = PaleoUnit("http://linked.earth/ontology/paleo_units#ng_g", "ng/g")
    peak_area = PaleoUnit("http://linked.earth/ontology/paleo_units#peak_area", "peak area")
    percent = PaleoUnit("http://linked.earth/ontology/paleo_units#percent", "percent")
    permil = PaleoUnit("http://linked.earth/ontology/paleo_units#permil", "permil")
    pH = PaleoUnit("http://linked.earth/ontology/paleo_units#pH", "pH")
    ppb = PaleoUnit("http://linked.earth/ontology/paleo_units#ppb", "ppb")
    ppm = PaleoUnit("http://linked.earth/ontology/paleo_units#ppm", "ppm")
    practical_salinity_unit = PaleoUnit("http://linked.earth/ontology/paleo_units#practical_salinity_unit", "practical salinity unit")
    ratio = PaleoUnit("http://linked.earth/ontology/paleo_units#ratio", "ratio")
    SI = PaleoUnit("http://linked.earth/ontology/paleo_units#SI", "SI")
    ug_cm2_yr = PaleoUnit("http://linked.earth/ontology/paleo_units#ug_cm2_yr", "ug/cm2/yr")
    ug_g = PaleoUnit("http://linked.earth/ontology/paleo_units#ug_g", "ug/g")
    um = PaleoUnit("http://linked.earth/ontology/paleo_units#um", "um")
    umol_mol = PaleoUnit("http://linked.earth/ontology/paleo_units#umol_mol", "umol/mol")
    unitless = PaleoUnit("http://linked.earth/ontology/paleo_units#unitless", "unitless")
    yr_14C_BP = PaleoUnit("http://linked.earth/ontology/paleo_units#yr_14C_BP", "yr 14C BP")
    yr_AD = PaleoUnit("http://linked.earth/ontology/paleo_units#yr_AD", "yr AD")
    yr_b2k = PaleoUnit("http://linked.earth/ontology/paleo_units#yr_b2k", "yr b2k")
    yr_BP = PaleoUnit("http://linked.earth/ontology/paleo_units#yr_BP", "yr BP")
    yr_ka = PaleoUnit("http://linked.earth/ontology/paleo_units#yr_ka", "yr ka")
    z_score = PaleoUnit("http://linked.earth/ontology/paleo_units#z_score", "z score")


from .synonyms import SYNONYMS

"""
The SCHEMA dictionary defines the conversion from and to RDF and LiPD
@fromJson refers to functions in LipdToRDF
@toJson refers to functions in RDFToLiPD
"""

SCHEMA = {
    'Dataset': {
        '@id': ['{dataSetName}'],
        '@toJson_pre': [
            '_set_archive_type_label'
        ],
        'datasetId': {
            'name': 'hasDatasetId'
        },
        'dataSetName': { 
            'name': 'hasName', 
            'alternates': ['paleoArchiveName'] 
        },
        'dataSource': { 
            'name': 'hasDataSource'
        },
        'originalDataURL': { 
            'name': 'hasOriginalDataUrl', 
            'alternates': ['originalDataUrl', 'additionalDataUrl', 'originalDataSource', 'originalDataURL', 'originalSourceUrl', 'paleoData_WDSPaleoUrl'] 
        },
        'dataContributor': {
            'name': 'hasContributor',
            'schema': 'Person',
            'alternates': ['whoEnteredinDB', 'MetadataEnteredByWhom', 'contributorName'],
            'fromJson': '_parse_persons'
        },
        'archiveType': {
            'name': 'hasArchiveType', 
            'alternates':[
                'archive',
                'paleoDataArchive',
                'paleoData_Archive',
                'Archive'
            ],
            'type': 'Individual',
            'synonyms': SYNONYMS['ARCHIVES']['ArchiveType'],
            'class_range': 'ArchiveType',
            'skip_auto_convert_to_json': True
        },
        'changelog': {
            'name': 'hasChangeLog',
            'schema': 'ChangeLog',
            'multiple': True
        },
        'notes': {
            'name': 'hasNotes'
        },
        'collectionName': {
            'name': 'hasCollectionName',
            'alternates': ['collectionName1', 'collectionName2', 'collectionName3']
        },
        'collectionYear': {
            'name': 'hasCollectionYear'
        },
        'investigator': {
            'name': 'hasInvestigator',
            'alternates': ['investigators'],
            'schema': 'Person',
            'multiple': True,
            'fromJson': '_parse_persons'
        },
        'creator': {
            'name': 'hasCreator',
            'schema': 'Person',
            'multiple': True,
            'fromJson': '_parse_persons'
        },
        'funding': { 
            'name': 'hasFunding', 
            'multiple': True, 
            'schema': 'Funding' 
        },
        'pub': { 
            'name': 'hasPublication', 
            'multiple': True, 
            'schema': 'Publication' 
        },
        'geo': {
            'name': 'hasLocation',
            'schema': 'Location',
            'fromJson': '_parse_location',
            'toJson': '_location_to_json'
        },
        'paleoData': {
            'name': 'hasPaleoData',
            'multiple': True,
            'schema': 'PaleoData'
        },
        'chronData': {
            'name': 'hasChronData',
            'multiple': True,
            'schema': 'ChronData'
        },
        'googleSpreadSheetKey': {
            'name': 'hasSpreadsheetLink',
            'fromJson': '_get_google_spreadsheet_url',
            'toJson': '_get_google_spreadsheet_key'
        },
        'dataSetVersion': { 
            'name': 'hasVersion' 
        },
        'compilation_nest': {
            'name': 'hasCompilationNest',
            'alternates': ['pages2kRegion', 'paleoDIVERSiteId', 'sisalSiteId', 'LegacyClimateDatasetId', 
                           'LegacyClimateSiteId', 'ch2kCoreCode', 'coralHydro2kGroup', 'iso2kCertification', 
                           'iso2kUI', 'ocean2kID', 'pages2kId', 'pages2kID', 'QCCertification', 'SISALEntityID' ]
        }
    },
    'Compilation': {
        '@id': ['{compilationName}', '.', '{@id}'],
        'compilationName': {
            'name': 'hasName'
        },
        'compilationVersion': {
            'name': 'hasVersion',
            'multiple': True
        }
    },
    'ChangeLog': {
        '@id': ['{@parent.@id}', '.ChangeLog.', '{@index}'],
        '@category': 'ChangeLog',
        'curator': {
            'name': 'hasCurator',
        },
        'version': {
            'name': 'hasVersion'
        },
        'lastVersion': {
            'name': 'hasLastVersion'
        },
        'timestamp': {
            'name': 'hasTimestamp'
        },
        'changes': {
            'name': 'hasChanges',
            'multiple': True,
            'type': 'Individual',
            'schema': 'Change',
            'fromJson': '_parse_changes',
            'toJson': '_changes_to_json'
        },
        'notes': {
            'name': 'hasNotes'
        }
    },
    'Change': {
        '@id': ['{@parent.@id}', '.Change.', '{@index}'],
        'name': {
            'name': 'hasName'
        },
        'notes': {
            'name': 'hasNotes',
            'multiple': True
        }
    },    
    'Funding': {
        '@id': [
            '{fundingAgency|agency}',
            '.',
            '{fundingGrant|grant}'
        ],
        'agency': { 
            'name': 'hasFundingAgency', 
            'alternates': ['fundingAgency'] 
        },
        'grant': {
            'name': 'hasGrant',
            'multiple': True,
            'alternates': ['fundingGrant']
        },
        'country': {
            'name': 'hasFundingCountry',
            'alternates': ['fundingCountry']
        },
        'investigator': {
            'name': 'hasInvestigator',
            'schema': 'Person',
            'multiple': True,
            'fromJson': '_parse_persons'
        }       
    },
    'Publication': {
        '@id': [
            'Publication.',
            '{identifier.0.id|@parent.dataSetName}',
            '{index}'
        ],
        # '@fromJson': ['_set_identifier_properties'],
        # '@toJson': ['_create_publication_identifier'],
        'title': { 
            'name': 'hasTitle' 
        },
        'abstract': { 
            'name': 'hasAbstract'
        },
        'institution': { 
            'name': 'hasInstitution'
        },
        'issue': { 
            'name': 'hasIssue'
        },
        'journal': { 
            'name': 'hasJournal'
        },    
        'volume': { 
            'name': 'hasVolume',
            'type': 'string'
        },
        'pages': { 
            'name': 'hasPages'
        },
        'year': { 
            'name': 'hasYear', 
            'type': 'integer',
            'alternates': ['pubYear'] 
        },        
        'publisher': { 
            'name': 'hasPublisher'
        },
        'report': { 
            'name': 'hasReport'
        },
        'type': { 
            'name': 'hasType'
        },
        'citation': { 
            'name': 'hasCitation', 
            'type': 'string'
        },
        'citeKey': { 
            'name': 'hasCiteKey', 
            'type': 'string'
        },
        'url': { 
            'name': 'hasUrl', 
            'alternates': ['link'],
            'multiple': True 
        },
        'dataUrl': { 
            'name': 'hasDataUrl', 
            'alternates': ['data_Url', 'pubDataUrl'],
            'multiple': True 
        },
        'doi': {
            'name': 'hasDOI',
            'type': 'string',
            'alternates': ['DOI']
        },
        'author': {
            'name': 'hasAuthor',
            'alternates': ['authors'],
            'schema': 'Person',
            'multiple': True,
            'fromJson': '_parse_persons'
        },
        'firstauthor': {
            'name': 'hasFirstAuthor',
            'alternates': ['firstAuthor'],
            'schema': 'Person',
            'fromJson': '_parse_persons'
        }
    },
    'PaleoData': {
        '@id': [
            '{@parent.dataSetName}',
            '.PaleoData',
            '{@index}'
        ],
        'paleoDataName': { 
            'name': 'hasName' 
        },
        'measurementTable': {
            'alternates': ['paleoMeasurementTable'],
            'name': 'hasMeasurementTable',
            'multiple': True,
            'schema': 'DataTable'
        },
        'model': {
            'alternates': ['paleoModel'],            
            'name': 'modeledBy',
            'multiple': True,
            'schema': 'Model'
        }
    },
    'ChronData': {
        '@id': [
            '{@parent.dataSetName}',
            '.ChronData',
            '{@index}'
        ],
        'measurementTable': {
            'alternates': ['chronMeasurementTable'],
            'name': 'hasMeasurementTable',
            'multiple': True,
            'schema': 'DataTable'
        },
        'model': {
            'alternates': ['chronModel'],            
            'name': 'modeledBy',
            'multiple': True,
            'schema': 'Model'
        }
    },
    'Model': {
        '@id': ['{@parent.@id}', '.Model', '{@index}'],
        'method': { 
            'name': 'hasCode'
        },
        'summaryTable': {
            'name': 'hasSummaryTable',
            'multiple': True,
            'schema': 'DataTable'
        },
        'ensembleTable': {
            'name': 'hasEnsembleTable',
            'multiple': True,
            'schema': 'DataTable'
        },
        'distributionTable': {
            'name': 'hasDistributionTable',
            'multiple': True,
            'schema': 'DataTable'
        }
    },    
    'DataTable': {
        '@id': ['{@parent.@id}', '.DataTable.', '{filename}', '_trunc(4)'],
        'filename': { 
            'name': 'hasFileName'
        },
        'columns': {
            'name': 'hasVariable',
            'multiple': True,
            'schema': 'Variable'
        },
        'missingValue': { 
            'name': 'hasMissingValue' 
        }
    },
    'Variable': {
        '@id': [
            '{foundInTable|@parent.@id}',
            '.',
            '{TSid|tsid|tSid}',
            '.',
            '{variableName|name}'
        ],
        '@fromJson': [
            '_wrap_uncertainty',
            '_add_found_in_table',
            '_add_found_in_dataset',
            '_add_variable_values',
            '_add_standard_variable',
            '_stringify_column_numbers_array'
        ],
        '@toJson_pre': [
            '_remove_found_in_table',
            '_remove_found_in_dataset',
            '_set_variable_name_from_standard_variable_label',
            '_set_units_label',
            '_set_proxy_label',
            '_set_archive_type_label',
            '_set_proxy_general_label'
        ],
        '@toJson': [
            '_unwrap_uncertainty',
            '_extract_variable_values',
            '_unarray_column_number'
        ],
        'number': { 
            'name': 'hasColumnNumber', 
            'type': 'integer'
        },
        'TSid': { 
            'name': 'hasVariableId', 
            'alternates': ['tsid', 'tSid'] 
        },
        'variableName': { 
            'name': 'hasName' 
        },
        'variableType': { 
            'name': 'hasType' 
        },
        'archiveType': {
            'name': 'hasArchiveType', 
            'alternates':[
                'archive',
                'paleoDataArchive',
                'paleoData_Archive',
                'Archive'
            ],
            'type': 'Individual',
            'synonyms': SYNONYMS['ARCHIVES']['ArchiveType'],
            'class_range': 'ArchiveType',
            'skip_auto_convert_to_json': True
        },
        'units': { 
            'name': 'hasUnits',
            'type': 'Individual',
            'synonyms': SYNONYMS['UNITS']['PaleoUnit'],
            'class_range': 'PaleoUnit',
            'skip_auto_convert_to_json': True
        },
        'missingValue': { 
            'name': 'hasMissingValue' 
        },
        'hasMaxValue': { 
            'name': 'hasMaxValue', 
            'alternates': ['hasMax'], 
            'type': 'float' 
        },
        'hasMinValue': { 
            'name': 'hasMinValue', 
            'alternates': ['hasMin'], 
            'type': 'float' 
        },
        'hasMeanValue': { 
            'name': 'hasMeanValue', 
            'alternates': ['hasMean'], 
            'type': 'float' 
        },
        'hasMedianValue': { 
            'name': 'hasMedianValue', 
            'alternates': ['hasMedian'], 
            'type': 'float' 
        },
        'description': {
            'name': 'hasDescription'
        },
        'isPrimary': {
            'name': 'isPrimary',
            'type': 'boolean'
        },
        'isComposite': {
            'name': 'isComposite',
            'type': 'boolean'
        },
        'measurementInstrument': {
            'name': 'hasInstrument',
            'type': 'Individual',
            'category': 'Instrument'
        },
        'calibration': {
            'name': 'calibratedVia',
            'schema': 'Calibration',
            'type': 'Individual',
            'multiple': True
        },
        'interpretation': {
            'name': 'hasInterpretation',
            'schema': 'Interpretation',
            'category': 'Interpretation',
            'type': 'Individual',
            'multiple': True
        },
        'resolution': {
            'name': 'hasResolution',
            'category': 'Resolution',
            'schema': 'Resolution',
            'type': 'Individual',
            'alternates': ['hasResolution']
        },
        'physicalSample': {
            'name': 'hasPhysicalSample',
            'schema': 'PhysicalSample',
            'category': 'PhysicalSample',
            'alternates': ['hasPhysicalSample'],
            'type': 'Individual',
            'multiple': True
        },
        'uncertainty': { 
            'name': 'hasUncertainty'         
        },
        'uncertaintyAnalytical': { 
            'name': 'hasUncertaintyAnalytical'
        },
        'uncertaintyReproducibility': { 
            'name': 'hasUncertaintyReproducibility'
        },
        'proxy': {
            'name': 'hasProxy',
            'type': 'Individual',
            'synonyms': SYNONYMS['PROXIES']['PaleoProxy'],
            'class_range': 'PaleoProxy',
            'skip_auto_convert_to_json': True
        },
        'proxyGeneral': {
            'name': 'hasProxyGeneral',
            'type': 'Individual',
            'synonyms': SYNONYMS['PROXIES']['PaleoProxyGeneral'],
            'class_range': 'PaleoProxyGeneral',
            'skip_auto_convert_to_json': True
        },
        'inCompilationBeta': {
            'name': 'partOfCompilation',
            'schema': 'Compilation',
            'category': 'Compilation',
            'type': 'Individual',
            'multiple': True            
        },
        'notes': {
            'name': 'hasNotes',
            'alternates': ['qcNotes', 'qCNotes', 'qCnotes', 'qcnotes', 'QCnotes', 'QCNotes']
        },
        'hasValues': {
            'type': 'string'
        },
        'foundInTable': {
            'type': 'Individual'
        },
        'foundInDataset': {
            'type': 'Individual'
        },
        'hasStandardVariable': {
            'type': 'EnumeratedIndividual',
            'synonyms': SYNONYMS["VARIABLES"]["PaleoVariable"],
            'class_range': 'PaleoVariable',
            'skip_auto_convert_to_json': True
        }        
    },
    'PhysicalSample': {
        'hasidentifier': { 
            'name': 'hasIGSN' 
        },
        'hasname': { 
            'name': 'name' 
        },
        'housedat': { 
            'name': 'housedAt' 
        }
    },
    'Resolution': {
        '@id': ['{@parent.@id}', '.Resolution'],
        '@toJson_pre': [
            '_set_units_label'
        ],
        'hasMaxValue': { 'name': 'hasMaxValue', 'alternates': ['hasMax'], 'type': 'float' },
        'hasMinValue': { 'name': 'hasMinValue', 'alternates': ['hasMin'], 'type': 'float' },
        'hasMeanValue': { 'name': 'hasMeanValue', 'alternates': ['hasMean'], 'type': 'float' },
        'hasMedianValue': { 'name': 'hasMedianValue', 'alternates': ['hasMedian'], 'type': 'float' },
        'units': { 
            'name': 'hasUnits',
            'type': 'Individual',
            'synonyms': SYNONYMS['UNITS']['PaleoUnit'],
            'class_range': 'PaleoUnit',
            'skip_auto_convert_to_json': True
        }
    },
    'Location': {
        '@id': ['{@parent.dataSetName}', '.Location'],
        'coordinates': { 
            'type': 'Geographic_coordinate',
            'class_type': 'string'
        },
        'coordinatesFor': { 
            'type': 'Individual' 
        },
        'type': { 'name': 'hasType' },
        'continent': { 'name': 'hasContinent' },
        'country': { 'name': 'hasCountry' },
        'countryOcean': { 'name': 'hasCountryOcean' },
        'description': { 'name': 'hasDescription' },
        'elevation': { 'name': 'hasElevation' },
        'geometryType': { 'name': 'hasGeometryType' },
        'latitude': { 'name': 'hasLatitude' },
        'longitude': { 'name': 'hasLongitude' },
        'locationName': { 'name': 'hasLocationName', 'alternates': ['secondarySiteName'] },
        'ocean': { 'name': 'hasOcean', 'alternates': ['ocean2'] },
        'siteName': { 'name': 'hasSiteName' },
        'notes': { 'name': 'hasNotes' }
    },
    'Interpretation': {
        '@id': [
            '{@parent.@id}',
            '.Interpretation',
            '{@index}'
        ],
        '@fromJson': ['_add_interpretation_rank'],
        '@toJson_pre': [
            '_set_units_label',
            '_set_seasonality_labels',
            '_set_interpretation_variable_label'
        ],        
        'variable': { 
            'name': 'hasVariable',
            'type': 'Individual',
            'synonyms': SYNONYMS['INTERPRETATION']['InterpretationVariable'],
            'class_range': 'InterpretationVariable',
            'skip_auto_convert_to_json': True
        },
        'variableGeneral': { 
            'name': 'hasVariableGeneral',
            'alternates': ['variableGroup']
        },
        'variableGeneralDirection': { 
            'name': 'hasVariableGeneralDirection',
            'alternates': ['variableGroupDirection'] 
        },
        'variableDetail': { 
            'name': 'hasVariableDetail', 
            'alternates': ['variabledetail'] 
        },        
        'seasonality': { 
            'name': 'hasSeasonality',
            'type': 'Individual',
            'synonyms': SYNONYMS['INTERPRETATION']['InterpretationSeasonality'],
            'class_range': 'InterpretationSeasonality',
            'skip_auto_convert_to_json': True
        },
        'seasonalityOriginal': { 
            'name': 'hasSeasonalityOriginal',
            'type': 'Individual',
            'synonyms': SYNONYMS['INTERPRETATION']['InterpretationSeasonality'],
            'class_range': 'InterpretationSeasonality',
            'skip_auto_convert_to_json': True
        },
        'seasonalityGeneral': { 
            'name': 'hasSeasonalityGeneral',
            'type': 'Individual',
            'synonyms': SYNONYMS['INTERPRETATION']['InterpretationSeasonality'],
            'class_range': 'InterpretationSeasonality',
            'skip_auto_convert_to_json': True
        },
        'notes': { 'name': 'hasNotes' },
        'rank': { 'name': 'hasRank' }, # TODO: Auto-create if it doesn't exist
        'basis': { 'name': 'hasBasis' },
        'scope': { 'name': 'hasScope' },
        'mathematicalRelation': { 'name': 'hasMathematicalRelation' },
        'direction': { 
            'name': 'hasDirection', 
            'alternates': ['interpDirection']
        },
        'isLocal': { 
            'name': 'isLocal', 
            'alternates': ['local']
        }
    },
    'Calibration': {
        '@id': ['{@parent.@id}', '.Calibration'],
        '@fromJson': ['_wrap_uncertainty'],
        '@toJson': ['_unwrap_uncertainty'],
        'datasetRange': {
            'name': 'hasDatasetRange'
        },
        'doi': {
            'name': 'hasDOI',
            'alternates': ['calibrationDOI', 'hasDOI', 'transferFunctionDOI']
        },
        'equation': {
            'name': 'hasEquation',
            'alternates': ['calibrationEquation']
        },
        'equationIntercept': {
            'name': 'hasEquationIntercept'
        },
        'equationR2': {
            'name': 'hasEquationR2'
        },
        'equationSlope': {
            'name': 'hasEquationSlope'
        },
        'equationSlopeUncertainty': {
            'name': 'hasEquationSlopeUncertainty'
        },
        'method': {
            'name': 'hasMethod'
        },
        'methodDetail': {
            'name': 'hasMethodDetail'
        },
        'proxyDataset': {
            'name': 'hasProxyDataset',
            'alternates': ['transferFunctionTrainingSet']
        },
        'targetDataset': {
            'name': 'hasTargetDataset',
            'alternates': ['target', 'dataset']
        },
        'hasSeasonality': {
            'name': 'seasonality',
            'alternates': ['transferFunctionTrainingSet']
        },
        'notes': {
            'name': 'hasNotes',
            'alternates': ['Note']
        },
        'uncertainty': { 
            'name': 'hasUncertainty',
            'alternates': ['uncertainty', 'calibrationUncertainty', 'temperature12kUncertainty', 'transferFunctionUncertainty'],
        }
    },
    'Person': { 
        '@id': ['{name}'],
        'name': {
            'name': 'hasName'
        }
    }
}



##############################
# Auto-generated. Do not Edit
##############################

import re
from pylipd.utils import uniqid
from pylipd.classes.person import Person

class Publication:
    """Auto-generated LinkedEarth class representing `Publication`."""
    def __init__(self):
        """Initialize a new Publication instance."""
        
        self.abstract: str = None
        self.authors: list[Person] = []
        self.citation: str = None
        self.citeKey: str = None
        self.dOI: str = None
        self.dataUrls: list[str] = []
        self.firstAuthor: Person = None
        self.institution: str = None
        self.issue: str = None
        self.journal: str = None
        self.pages: str = None
        self.publicationType: str = None
        self.publisher: str = None
        self.report: str = None
        self.title: str = None
        self.urls: list[str] = []
        self.volume: str = None
        self.year: int = None
        self.misc = {}
        self.ontns = "http://linked.earth/ontology#"
        self.ns = "http://linked.earth/lipd"
        self.type = "http://linked.earth/ontology#Publication"
        self.id = self.ns + "/" + uniqid("Publication.")

    @staticmethod
    def from_data(id, data) -> 'Publication':
        """Instantiate `Publication` from an ontology-style data graph.

        Parameters
        ----------
        id : str
            The node identifier for this object.
        data : dict
            Dictionary mapping node ids to their predicate lists.

        Returns
        -------
        Publication
            The populated `Publication` instance.
        """
        self = Publication()
        self.id = id
        mydata = data[id]
        for key in mydata:
            value = mydata[key]
            obj = None
            if key == "type":
                for val in value:
                    self.type = val["@id"]
        
            elif key == "hasAbstract":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.abstract = obj
        
            elif key == "hasAuthor":
                for val in value:
                    if "@id" in val:
                        obj = Person.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
            
                    self.authors.append(obj)
        
            elif key == "hasCitation":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.citation = obj
        
            elif key == "hasCiteKey":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.citeKey = obj
        
            elif key == "hasDOI":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.dOI = obj
        
            elif key == "hasDataUrl":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]
                    self.dataUrls.append(obj)
        
            elif key == "hasFirstAuthor":
                for val in value:
                    if "@id" in val:
                        obj = Person.from_data(val["@id"], data)
                    else:
                        obj = val["@value"]
                                    
                    self.firstAuthor = obj
        
            elif key == "hasInstitution":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.institution = obj
        
            elif key == "hasIssue":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.issue = obj
        
            elif key == "hasJournal":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.journal = obj
        
            elif key == "hasPages":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.pages = obj
        
            elif key == "hasPublisher":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.publisher = obj
        
            elif key == "hasReport":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.report = obj
        
            elif key == "hasTitle":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.title = obj
        
            elif key == "hasType":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.publicationType = obj
        
            elif key == "hasUrl":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]
                    self.urls.append(obj)
        
            elif key == "hasVolume":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.volume = obj
        
            elif key == "hasYear":
                for val in value:
                    if "@value" in val:
                        obj = val["@value"]                        
                    self.year = obj
            else:
                for val in value:
                    obj = None
                    if "@id" in val:
                        obj = data[val["@id"]]
                    elif "@value" in val:
                        obj = val["@value"]
                    self.set_non_standard_property(key, obj)
            
        return self

    def to_data(self, data={}):
        """Serialize the object into a JSON-LD compatible dictionary.

        Parameters
        ----------
        data : dict, optional
            Existing data dictionary to extend.

        Returns
        -------
        dict
            The updated data dictionary.
        """
        data[self.id] = {}
        data[self.id]["type"] = [
            {
                "@id": self.type,
                "@type": "uri"
            }
        ]

        if len(self.authors):
            data[self.id]["hasAuthor"] = []
        for value_obj in self.authors:
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasAuthor"].append(obj)

        if len(self.dataUrls):
            data[self.id]["hasDataUrl"] = []
        for value_obj in self.dataUrls:
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasDataUrl"].append(obj)

        if len(self.urls):
            data[self.id]["hasUrl"] = []
        for value_obj in self.urls:
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasUrl"].append(obj)

        if self.abstract:
            value_obj = self.abstract
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasAbstract"] = [obj]
                

        if self.citation:
            value_obj = self.citation
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasCitation"] = [obj]
                

        if self.citeKey:
            value_obj = self.citeKey
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasCiteKey"] = [obj]
                

        if self.dOI:
            value_obj = self.dOI
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasDOI"] = [obj]
                

        if self.firstAuthor:
            value_obj = self.firstAuthor
            if type(value_obj) is str:
                obj = {
                    "@value": value_obj,
                    "@type": "literal",
                    "@datatype": "http://www.w3.org/2001/XMLSchema#string"
                }
            else:
                obj = {
                    "@id": value_obj.id,
                    "@type": "uri"
                }
                data = value_obj.to_data(data)
            data[self.id]["hasFirstAuthor"] = [obj]
                

        if self.institution:
            value_obj = self.institution
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasInstitution"] = [obj]
                

        if self.issue:
            value_obj = self.issue
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasIssue"] = [obj]
                

        if self.journal:
            value_obj = self.journal
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasJournal"] = [obj]
                

        if self.pages:
            value_obj = self.pages
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasPages"] = [obj]
                

        if self.publicationType:
            value_obj = self.publicationType
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasType"] = [obj]
                

        if self.publisher:
            value_obj = self.publisher
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasPublisher"] = [obj]
                

        if self.report:
            value_obj = self.report
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasReport"] = [obj]
                

        if self.title:
            value_obj = self.title
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasTitle"] = [obj]
                

        if self.volume:
            value_obj = self.volume
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#string"
            }
            data[self.id]["hasVolume"] = [obj]
                

        if self.year:
            value_obj = self.year
            obj = {
                "@value": value_obj,
                "@type": "literal",
                "@datatype": "http://www.w3.org/2001/XMLSchema#integer"
            }
            data[self.id]["hasYear"] = [obj]
                
        for key in self.misc:
            value = self.misc[key]
            data[self.id][key] = []
            ptype = None
            tp = type(value).__name__
            if tp == "int":
                ptype = "http://www.w3.org/2001/XMLSchema#integer"
            elif tp == "float" or tp == "double":
                ptype = "http://www.w3.org/2001/XMLSchema#float"
            elif tp == "str":
                if re.match(r"\d{4}-\d{2}-\d{2}( |T)\d{2}:\d{2}:\d{2}", value):
                    ptype = "http://www.w3.org/2001/XMLSchema#datetime"   
                elif re.match(r"\d{4}-\d{2}-\d{2}", value):
                    ptype = "http://www.w3.org/2001/XMLSchema#date"
                else:
                    ptype = "http://www.w3.org/2001/XMLSchema#string"
            elif tp == "bool":
                ptype = "http://www.w3.org/2001/XMLSchema#boolean"

            data[self.id][key].append({
                "@value": value,
                "@type": "literal",
                "@datatype": ptype
            })
        
        return data

    def to_json(self):
        """Return a lightweight JSON representation (used by LiPD).

        Returns
        -------
        dict
            A dictionary representation of this object.
        """
        data = {
            "@id": self.id
        }

        if len(self.authors):
            data["author"] = []
        for value_obj in self.authors:
            obj = value_obj.to_json()
            data["author"].append(obj)

        if len(self.dataUrls):
            data["dataUrl"] = []
        for value_obj in self.dataUrls:
            obj = value_obj
            data["dataUrl"].append(obj)

        if len(self.urls):
            data["url"] = []
        for value_obj in self.urls:
            obj = value_obj
            data["url"].append(obj)

        if self.abstract:
            value_obj = self.abstract
            obj = value_obj
            data["abstract"] = obj

        if self.citation:
            value_obj = self.citation
            obj = value_obj
            data["citation"] = obj

        if self.citeKey:
            value_obj = self.citeKey
            obj = value_obj
            data["citeKey"] = obj

        if self.dOI:
            value_obj = self.dOI
            obj = value_obj
            data["doi"] = obj

        if self.firstAuthor:
            value_obj = self.firstAuthor
            obj = value_obj.to_json()
            data["firstauthor"] = obj

        if self.institution:
            value_obj = self.institution
            obj = value_obj
            data["institution"] = obj

        if self.issue:
            value_obj = self.issue
            obj = value_obj
            data["issue"] = obj

        if self.journal:
            value_obj = self.journal
            obj = value_obj
            data["journal"] = obj

        if self.pages:
            value_obj = self.pages
            obj = value_obj
            data["pages"] = obj

        if self.publicationType:
            value_obj = self.publicationType
            obj = value_obj
            data["type"] = obj

        if self.publisher:
            value_obj = self.publisher
            obj = value_obj
            data["publisher"] = obj

        if self.report:
            value_obj = self.report
            obj = value_obj
            data["report"] = obj

        if self.title:
            value_obj = self.title
            obj = value_obj
            data["title"] = obj

        if self.volume:
            value_obj = self.volume
            obj = value_obj
            data["volume"] = obj

        if self.year:
            value_obj = self.year
            obj = value_obj
            data["year"] = obj

        for key in self.misc:
            value = self.misc[key]
            data[key] = value
                   
        return data

    @staticmethod
    def from_json(data) -> 'Publication':
        """Instantiate `Publication` from its lightweight JSON representation.

        Parameters
        ----------
        data : dict
            The JSON dictionary to parse.

        Returns
        -------
        Publication
            The populated `Publication` instance.
        """
        self = Publication()
        for key in data:
            pvalue = data[key]
            if key == "@id":
                self.id = pvalue
            elif key == "abstract":
                    value = pvalue
                    obj = value
                    self.abstract = obj
            elif key == "author":
                for value in pvalue:
                    obj = Person.from_json(value)
                    self.authors.append(obj)
            elif key == "citation":
                    value = pvalue
                    obj = value
                    self.citation = obj
            elif key == "citeKey":
                    value = pvalue
                    obj = value
                    self.citeKey = obj
            elif key == "dataUrl":
                for value in pvalue:
                    obj = value
                    self.dataUrls.append(obj)
            elif key == "doi":
                    value = pvalue
                    obj = value
                    self.dOI = obj
            elif key == "firstauthor":
                    value = pvalue
                    obj = Person.from_json(value)
                    self.firstAuthor = obj
            elif key == "institution":
                    value = pvalue
                    obj = value
                    self.institution = obj
            elif key == "issue":
                    value = pvalue
                    obj = value
                    self.issue = obj
            elif key == "journal":
                    value = pvalue
                    obj = value
                    self.journal = obj
            elif key == "pages":
                    value = pvalue
                    obj = value
                    self.pages = obj
            elif key == "publisher":
                    value = pvalue
                    obj = value
                    self.publisher = obj
            elif key == "report":
                    value = pvalue
                    obj = value
                    self.report = obj
            elif key == "title":
                    value = pvalue
                    obj = value
                    self.title = obj
            elif key == "type":
                    value = pvalue
                    obj = value
                    self.publicationType = obj
            elif key == "url":
                for value in pvalue:
                    obj = value
                    self.urls.append(obj)
            elif key == "volume":
                    value = pvalue
                    obj = value
                    self.volume = obj
            elif key == "year":
                    value = pvalue
                    obj = value
                    self.year = obj
            else:
                self.set_non_standard_property(key, pvalue)
                   
        return self

    def set_non_standard_property(self, key, value):
        """Store a predicate that is not defined in the ontology schema.

        This is useful for forward-compatibility with new properties that are
        not yet part of the official schema.

        Parameters
        ----------
        key : str
            The property name.
        value : any
            The property value.
        """
        if key not in self.misc:
            self.misc[key] = value
    
    def get_non_standard_property(self, key):
        """Return a single non-standard property by key.

        Parameters
        ----------
        key : str
            The property name.

        Returns
        -------
        any
            The property value.
        """
        return self.misc[key]
                
    def get_all_non_standard_properties(self):
        """Return the dictionary of all non-standard properties.

        Returns
        -------
        dict
            Dictionary of all non-standard properties.
        """
        return self.misc

    def add_non_standard_property(self, key, value):
        """Append a value to a list-valued non-standard property.

        Parameters
        ----------
        key : str
            The property name.
        value : any
            The value to append.
        """
        if key not in self.misc:
            self.misc[key] = []
        self.misc[key].append(value)
        
    def getAbstract(self) -> str:
        """Get abstract.

        Returns
        -------
        str
            The current value of abstract.
        """
        return self.abstract

    def setAbstract(self, abstract:str):
        """Set abstract.

        Parameters
        ----------
        abstract : str
            The value to assign.
        """
        assert isinstance(abstract, str), f"Error: '{abstract}' is not of type str"
        self.abstract = abstract
    
    def getAuthors(self) -> list[Person]:
        """Get authors list.

        Returns
        -------
        list[Person]
            A list of Person objects.
        """
        return self.authors

    def setAuthors(self, authors:list[Person]):
        """Set the authors list.

        Parameters
        ----------
        authors : list[Person]
            The list to assign.
        """
        assert isinstance(authors, list), "Error: authors is not a list"
        assert all(isinstance(x, Person) for x in authors), f"Error: '{authors}' is not of type Person"
        self.authors = authors

    def addAuthor(self, authors:Person):
        """Add a value to the authors list.

        Parameters
        ----------
        authors : Person
            The value to append.
        """
        assert isinstance(authors, Person), f"Error: '{authors}' is not of type Person"
        self.authors.append(authors)
        
    def getCitation(self) -> str:
        """Get citation.

        Returns
        -------
        str
            The current value of citation.
        """
        return self.citation

    def setCitation(self, citation:str):
        """Set citation.

        Parameters
        ----------
        citation : str
            The value to assign.
        """
        assert isinstance(citation, str), f"Error: '{citation}' is not of type str"
        self.citation = citation
    
    def getCiteKey(self) -> str:
        """Get citeKey.

        Returns
        -------
        str
            The current value of citeKey.
        """
        return self.citeKey

    def setCiteKey(self, citeKey:str):
        """Set citeKey.

        Parameters
        ----------
        citeKey : str
            The value to assign.
        """
        assert isinstance(citeKey, str), f"Error: '{citeKey}' is not of type str"
        self.citeKey = citeKey
    
    def getDOI(self) -> str:
        """Get dOI.

        Returns
        -------
        str
            The current value of dOI.
        """
        return self.dOI

    def setDOI(self, dOI:str):
        """Set dOI.

        Parameters
        ----------
        dOI : str
            The value to assign.
        """
        assert isinstance(dOI, str), f"Error: '{dOI}' is not of type str"
        self.dOI = dOI
    
    def getDataUrls(self) -> list[str]:
        """Get dataUrls list.

        Returns
        -------
        list[str]
            A list of str objects.
        """
        return self.dataUrls

    def setDataUrls(self, dataUrls:list[str]):
        """Set the dataUrls list.

        Parameters
        ----------
        dataUrls : list[str]
            The list to assign.
        """
        assert isinstance(dataUrls, list), "Error: dataUrls is not a list"
        assert all(isinstance(x, str) for x in dataUrls), f"Error: '{dataUrls}' is not of type str"
        self.dataUrls = dataUrls

    def addDataUrl(self, dataUrls:str):
        """Add a value to the dataUrls list.

        Parameters
        ----------
        dataUrls : str
            The value to append.
        """
        assert isinstance(dataUrls, str), f"Error: '{dataUrls}' is not of type str"
        self.dataUrls.append(dataUrls)
        
    def getFirstAuthor(self) -> Person:
        """Get firstAuthor.

        Returns
        -------
        Person
            The current value of firstAuthor.
        """
        return self.firstAuthor

    def setFirstAuthor(self, firstAuthor:Person):
        """Set firstAuthor.

        Parameters
        ----------
        firstAuthor : Person
            The value to assign.
        """
        assert isinstance(firstAuthor, Person), f"Error: '{firstAuthor}' is not of type Person"
        self.firstAuthor = firstAuthor
    
    def getInstitution(self) -> str:
        """Get institution.

        Returns
        -------
        str
            The current value of institution.
        """
        return self.institution

    def setInstitution(self, institution:str):
        """Set institution.

        Parameters
        ----------
        institution : str
            The value to assign.
        """
        assert isinstance(institution, str), f"Error: '{institution}' is not of type str"
        self.institution = institution
    
    def getIssue(self) -> str:
        """Get issue.

        Returns
        -------
        str
            The current value of issue.
        """
        return self.issue

    def setIssue(self, issue:str):
        """Set issue.

        Parameters
        ----------
        issue : str
            The value to assign.
        """
        assert isinstance(issue, str), f"Error: '{issue}' is not of type str"
        self.issue = issue
    
    def getJournal(self) -> str:
        """Get journal.

        Returns
        -------
        str
            The current value of journal.
        """
        return self.journal

    def setJournal(self, journal:str):
        """Set journal.

        Parameters
        ----------
        journal : str
            The value to assign.
        """
        assert isinstance(journal, str), f"Error: '{journal}' is not of type str"
        self.journal = journal
    
    def getPages(self) -> str:
        """Get pages.

        Returns
        -------
        str
            The current value of pages.
        """
        return self.pages

    def setPages(self, pages:str):
        """Set pages.

        Parameters
        ----------
        pages : str
            The value to assign.
        """
        assert isinstance(pages, str), f"Error: '{pages}' is not of type str"
        self.pages = pages
    
    def getPublicationType(self) -> str:
        """Get publicationType.

        Returns
        -------
        str
            The current value of publicationType.
        """
        return self.publicationType

    def setPublicationType(self, publicationType:str):
        """Set publicationType.

        Parameters
        ----------
        publicationType : str
            The value to assign.
        """
        assert isinstance(publicationType, str), f"Error: '{publicationType}' is not of type str"
        self.publicationType = publicationType
    
    def getPublisher(self) -> str:
        """Get publisher.

        Returns
        -------
        str
            The current value of publisher.
        """
        return self.publisher

    def setPublisher(self, publisher:str):
        """Set publisher.

        Parameters
        ----------
        publisher : str
            The value to assign.
        """
        assert isinstance(publisher, str), f"Error: '{publisher}' is not of type str"
        self.publisher = publisher
    
    def getReport(self) -> str:
        """Get report.

        Returns
        -------
        str
            The current value of report.
        """
        return self.report

    def setReport(self, report:str):
        """Set report.

        Parameters
        ----------
        report : str
            The value to assign.
        """
        assert isinstance(report, str), f"Error: '{report}' is not of type str"
        self.report = report
    
    def getTitle(self) -> str:
        """Get title.

        Returns
        -------
        str
            The current value of title.
        """
        return self.title

    def setTitle(self, title:str):
        """Set title.

        Parameters
        ----------
        title : str
            The value to assign.
        """
        assert isinstance(title, str), f"Error: '{title}' is not of type str"
        self.title = title
    
    def getUrls(self) -> list[str]:
        """Get urls list.

        Returns
        -------
        list[str]
            A list of str objects.
        """
        return self.urls

    def setUrls(self, urls:list[str]):
        """Set the urls list.

        Parameters
        ----------
        urls : list[str]
            The list to assign.
        """
        assert isinstance(urls, list), "Error: urls is not a list"
        assert all(isinstance(x, str) for x in urls), f"Error: '{urls}' is not of type str"
        self.urls = urls

    def addUrl(self, urls:str):
        """Add a value to the urls list.

        Parameters
        ----------
        urls : str
            The value to append.
        """
        assert isinstance(urls, str), f"Error: '{urls}' is not of type str"
        self.urls.append(urls)
        
    def getVolume(self) -> str:
        """Get volume.

        Returns
        -------
        str
            The current value of volume.
        """
        return self.volume

    def setVolume(self, volume:str):
        """Set volume.

        Parameters
        ----------
        volume : str
            The value to assign.
        """
        assert isinstance(volume, str), f"Error: '{volume}' is not of type str"
        self.volume = volume
    
    def getYear(self) -> int:
        """Get year.

        Returns
        -------
        int
            The current value of year.
        """
        return self.year

    def setYear(self, year:int):
        """Set year.

        Parameters
        ----------
        year : int
            The value to assign.
        """
        assert isinstance(year, int), f"Error: '{year}' is not of type int"
        self.year = year
    


from pylipd.utils.dataset import available_dataset_names, load_datasets, load_dir
import pytest 
import random

@pytest.fixture
def odp846():
    D = load_datasets('ODP846.Lawrence.2006')
    return D

@pytest.fixture
def multipleLipds(seed = 20):
    names = available_dataset_names()
    random.seed(seed)
    name_rand = random.sample(names,2)
    D = load_datasets(name_rand)
    return D, name_rand

@pytest.fixture
def pages2k():
    D = load_dir('Pages2k')
    return D

@pytest.fixture
def temp12k():
    D = load_dir('Temp12k')
    return D




##############################
# Auto-generated. Do not Edit
##############################
from pylipd.globals.synonyms import SYNONYMS

class PaleoVariable:
    """Controlled-vocabulary class for `PaleoVariable` terms."""
    synonyms = SYNONYMS["VARIABLES"]["PaleoVariable"]

    def __init__(self, id, label):
        """Initialize a PaleoVariable term.

        Parameters
        ----------
        id : str
            The full URI identifier for this term.
        label : str
            The human-readable label for this term.
        """
        self.id = id
        self.label = label
    
    def __eq__(self, value: object) -> bool:
        """Check equality based on term ID.

        Parameters
        ----------
        value : object
            The object to compare against.

        Returns
        -------
        bool
            True if the IDs match, False otherwise.
        """
        return self.id == value.id
        
    def getLabel(self):
        """Return the human-readable label of the term.

        Returns
        -------
        str
            The label for this term.
        """
        return self.label

    def getId(self):
        """Return the full URI identifier of the term.

        Returns
        -------
        str
            The URI identifier for this term.
        """
        return self.id
    
    def to_data(self, data={}):
        """Serialize this term to JSON-LD format.

        Parameters
        ----------
        data : dict, optional
            Existing data dictionary to extend.

        Returns
        -------
        dict
            The updated data dictionary.
        """
        data[self.id] ={
            "label": [
                {
                    "@datatype": None,
                    "@type": "literal",
                    "@value": self.label
                }
            ]
        }
        return data

    def to_json(self):
        """Return the plain-text label (used in lightweight JSON).

        Returns
        -------
        str
            The label for this term.
        """
        data = self.label
        return data

    @classmethod
    def from_synonym(cls, synonym):
        """Create a PaleoVariable instance from a synonym string.

        Parameters
        ----------
        synonym : str
            A synonym or alternative name for the term.

        Returns
        -------
        PaleoVariable or None
            The PaleoVariable instance if found, None otherwise.
        """
        if synonym.lower() in PaleoVariable.synonyms:
            synobj = PaleoVariable.synonyms[synonym.lower()]
            return PaleoVariable(synobj['id'], synobj['label'])
        return None
        
class PaleoVariableConstants:
    ACL = PaleoVariable("http://linked.earth/ontology/paleo_variables#ACL", "ACL")
    AET_PET = PaleoVariable("http://linked.earth/ontology/paleo_variables#AET_PET", "AET/PET")
    ARM_IRM = PaleoVariable("http://linked.earth/ontology/paleo_variables#ARM_IRM", "ARM/IRM")
    ARSTAN = PaleoVariable("http://linked.earth/ontology/paleo_variables#ARSTAN", "ARSTAN")
    Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Al", "Al")
    Al2O3 = PaleoVariable("http://linked.earth/ontology/paleo_variables#Al2O3", "Al2O3")
    As = PaleoVariable("http://linked.earth/ontology/paleo_variables#As", "As")
    BIT = PaleoVariable("http://linked.earth/ontology/paleo_variables#BIT", "BIT")
    BSi = PaleoVariable("http://linked.earth/ontology/paleo_variables#BSi", "BSi")
    Ba = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ba", "Ba")
    Ba_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ba_Al", "Ba/Al")
    Ba_Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ba_Ca", "Ba/Ca")
    Be = PaleoVariable("http://linked.earth/ontology/paleo_variables#Be", "Be")
    Br = PaleoVariable("http://linked.earth/ontology/paleo_variables#Br", "Br")
    C20n_alkenoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C20n-alkenoicAcid", "C20n-alkenoicAcid")
    C21n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C21n-alkanoicAcid", "C21n-alkanoicAcid")
    C22n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C22n-alkanoicAcid", "C22n-alkanoicAcid")
    C23n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C23n-alkanoicAcid", "C23n-alkanoicAcid")
    C24n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C24n-alkanoicAcid", "C24n-alkanoicAcid")
    C25_2n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C25_2n-alkanoicAcid", "C25_2n-alkanoicAcid")
    C25n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C25n-alkanoicAcid", "C25n-alkanoicAcid")
    C26n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C26n-alkanoicAcid", "C26n-alkanoicAcid")
    C27n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C27n-alkanoicAcid", "C27n-alkanoicAcid")
    C28n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C28n-alkanoicAcid", "C28n-alkanoicAcid")
    C29n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C29n-alkanoicAcid", "C29n-alkanoicAcid")
    C30n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C30n-alkanoicAcid", "C30n-alkanoicAcid")
    C31n_alkanoicAcid = PaleoVariable("http://linked.earth/ontology/paleo_variables#C31n-alkanoicAcid", "C31n-alkanoicAcid")
    C37Alkenone = PaleoVariable("http://linked.earth/ontology/paleo_variables#C37Alkenone", "C37Alkenone")
    C37_2Alkenone = PaleoVariable("http://linked.earth/ontology/paleo_variables#C37_2Alkenone", "C37:2Alkenone")
    C37_3aAlkenone = PaleoVariable("http://linked.earth/ontology/paleo_variables#C37_3aAlkenone", "C37:3aAlkenone")
    C37_3bAlkenone = PaleoVariable("http://linked.earth/ontology/paleo_variables#C37_3bAlkenone", "C37:3bAlkenone")
    C37_4Alkenone = PaleoVariable("http://linked.earth/ontology/paleo_variables#C37_4Alkenone", "C37:4Alkenone")
    CBT = PaleoVariable("http://linked.earth/ontology/paleo_variables#CBT", "CBT")
    CCA1 = PaleoVariable("http://linked.earth/ontology/paleo_variables#CCA1", "CCA1")
    CCA2 = PaleoVariable("http://linked.earth/ontology/paleo_variables#CCA2", "CCA2")
    CPI = PaleoVariable("http://linked.earth/ontology/paleo_variables#CPI", "CPI")
    C_N = PaleoVariable("http://linked.earth/ontology/paleo_variables#C_N", "C/N")
    Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ca", "Ca")
    CaCO3 = PaleoVariable("http://linked.earth/ontology/paleo_variables#CaCO3", "CaCO3")
    CaO = PaleoVariable("http://linked.earth/ontology/paleo_variables#CaO", "CaO")
    Ca_K = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ca_K", "Ca/K")
    Ca_Sr = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ca_Sr", "Ca/Sr")
    Ca_Ti = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ca_Ti", "Ca/Ti")
    Ti_Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ti_Ca", "Ti/Ca")
    Cd = PaleoVariable("http://linked.earth/ontology/paleo_variables#Cd", "Cd")
    Cd_Mn = PaleoVariable("http://linked.earth/ontology/paleo_variables#Cd_Mn", "Cd/Mn")
    Cl = PaleoVariable("http://linked.earth/ontology/paleo_variables#Cl", "Cl")
    Co = PaleoVariable("http://linked.earth/ontology/paleo_variables#Co", "Co")
    Cr = PaleoVariable("http://linked.earth/ontology/paleo_variables#Cr", "Cr")
    Cu = PaleoVariable("http://linked.earth/ontology/paleo_variables#Cu", "Cu")
    DWHI = PaleoVariable("http://linked.earth/ontology/paleo_variables#DWHI", "DWHI")
    Dd2H = PaleoVariable("http://linked.earth/ontology/paleo_variables#Dd2H", "Dd2H")
    EPS = PaleoVariable("http://linked.earth/ontology/paleo_variables#EPS", "EPS")
    ElNinoEvent = PaleoVariable("http://linked.earth/ontology/paleo_variables#ElNinoEvent", "ElNinoEvent")
    Eu_Zr = PaleoVariable("http://linked.earth/ontology/paleo_variables#Eu_Zr", "Eu/Zr")
    Fe = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe", "Fe")
    Fe2O3 = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe2O3", "Fe2O3")
    Fe_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe_Al", "Fe/Al")
    Fe_Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe_Ca", "Fe/Ca")
    Fe_K = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe_K", "Fe/K")
    Fe_Mn = PaleoVariable("http://linked.earth/ontology/paleo_variables#Fe_Mn", "Fe/Mn")
    GDGT = PaleoVariable("http://linked.earth/ontology/paleo_variables#GDGT", "GDGT")
    GDGT_0_Cren = PaleoVariable("http://linked.earth/ontology/paleo_variables#GDGT-0_Cren", "GDGT-0/Cren")
    IP25 = PaleoVariable("http://linked.earth/ontology/paleo_variables#IP25", "IP25")
    IRM = PaleoVariable("http://linked.earth/ontology/paleo_variables#IRM", "IRM")
    ITCZ = PaleoVariable("http://linked.earth/ontology/paleo_variables#ITCZ", "ITCZ")
    JulianDay = PaleoVariable("http://linked.earth/ontology/paleo_variables#JulianDay", "JulianDay")
    K2O = PaleoVariable("http://linked.earth/ontology/paleo_variables#K2O", "K2O")
    K37 = PaleoVariable("http://linked.earth/ontology/paleo_variables#K37", "K37")
    K_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#K_Al", "K/Al")
    LDI = PaleoVariable("http://linked.earth/ontology/paleo_variables#LDI", "LDI")
    LOI = PaleoVariable("http://linked.earth/ontology/paleo_variables#LOI", "LOI")
    La = PaleoVariable("http://linked.earth/ontology/paleo_variables#La", "La")
    MAR = PaleoVariable("http://linked.earth/ontology/paleo_variables#MAR", "MAR")
    MBT = PaleoVariable("http://linked.earth/ontology/paleo_variables#MBT", "MBT")
    MS = PaleoVariable("http://linked.earth/ontology/paleo_variables#MS", "MS")
    Si = PaleoVariable("http://linked.earth/ontology/paleo_variables#Si", "Si")
    MXD = PaleoVariable("http://linked.earth/ontology/paleo_variables#MXD", "MXD")
    Mg = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mg", "Mg")
    MgO = PaleoVariable("http://linked.earth/ontology/paleo_variables#MgO", "MgO")
    Mg_Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mg_Ca", "Mg/Ca")
    Mn = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mn", "Mn")
    MnO = PaleoVariable("http://linked.earth/ontology/paleo_variables#MnO", "MnO")
    Mn_Fe = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mn_Fe", "Mn/Fe")
    Mn_Mo = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mn_Mo", "Mn/Mo")
    Mo = PaleoVariable("http://linked.earth/ontology/paleo_variables#Mo", "Mo")
    NO3 = PaleoVariable("http://linked.earth/ontology/paleo_variables#NO3", "NO3")
    nitrate = PaleoVariable("http://linked.earth/ontology/paleo_variables#nitrate", "nitrate")
    N_C = PaleoVariable("http://linked.earth/ontology/paleo_variables#N_C", "N/C")
    Na2O = PaleoVariable("http://linked.earth/ontology/paleo_variables#Na2O", "Na2O")
    Ni = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ni", "Ni")
    PC1 = PaleoVariable("http://linked.earth/ontology/paleo_variables#PC1", "PC1")
    PC3 = PaleoVariable("http://linked.earth/ontology/paleo_variables#PC3", "PC3")
    PC2 = PaleoVariable("http://linked.earth/ontology/paleo_variables#PC2", "PC2")
    Paq = PaleoVariable("http://linked.earth/ontology/paleo_variables#Paq", "Paq")
    Pb = PaleoVariable("http://linked.earth/ontology/paleo_variables#Pb", "Pb")
    Picea_Artemisia = PaleoVariable("http://linked.earth/ontology/paleo_variables#Picea_Artemisia", "Picea/Artemisia")
    Picea_Pinus = PaleoVariable("http://linked.earth/ontology/paleo_variables#Picea_Pinus", "Picea/Pinus")
    Pinus_Artemisia = PaleoVariable("http://linked.earth/ontology/paleo_variables#Pinus_Artemisia", "Pinus/Artemisia")
    Poaceae_Ephedra = PaleoVariable("http://linked.earth/ontology/paleo_variables#Poaceae_Ephedra", "Poaceae/Ephedra")
    R570_R630 = PaleoVariable("http://linked.earth/ontology/paleo_variables#R570_R630", "R570/R630")
    R650_R700 = PaleoVariable("http://linked.earth/ontology/paleo_variables#R650_R700", "R650/R700")
    RABD660670 = PaleoVariable("http://linked.earth/ontology/paleo_variables#RABD660670", "RABD660670")
    RAN15 = PaleoVariable("http://linked.earth/ontology/paleo_variables#RAN15", "RAN15")
    RBAR = PaleoVariable("http://linked.earth/ontology/paleo_variables#RBAR", "RBAR")
    Rb = PaleoVariable("http://linked.earth/ontology/paleo_variables#Rb", "Rb")
    Rb87_Sr86 = PaleoVariable("http://linked.earth/ontology/paleo_variables#Rb87_Sr86", "Rb87/Sr86")
    SO4 = PaleoVariable("http://linked.earth/ontology/paleo_variables#SO4", "SO4")
    sulfate = PaleoVariable("http://linked.earth/ontology/paleo_variables#sulfate", "sulfate")
    salinity = PaleoVariable("http://linked.earth/ontology/paleo_variables#salinity", "salinity")
    Sc = PaleoVariable("http://linked.earth/ontology/paleo_variables#Sc", "Sc")
    Si_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Si_Al", "Si/Al")
    Si_Ti = PaleoVariable("http://linked.earth/ontology/paleo_variables#Si_Ti", "Si/Ti")
    Sr = PaleoVariable("http://linked.earth/ontology/paleo_variables#Sr", "Sr")
    Sr_Ca = PaleoVariable("http://linked.earth/ontology/paleo_variables#Sr_Ca", "Sr/Ca")
    TDS = PaleoVariable("http://linked.earth/ontology/paleo_variables#TDS", "TDS")
    TEX86 = PaleoVariable("http://linked.earth/ontology/paleo_variables#TEX86", "TEX86")
    TIC = PaleoVariable("http://linked.earth/ontology/paleo_variables#TIC", "TIC")
    TOC = PaleoVariable("http://linked.earth/ontology/paleo_variables#TOC", "TOC")
    organicCarbon = PaleoVariable("http://linked.earth/ontology/paleo_variables#organicCarbon", "organicCarbon")
    TOC_TN = PaleoVariable("http://linked.earth/ontology/paleo_variables#TOC_TN", "TOC/TN")
    Ti = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ti", "Ti")
    TiO2 = PaleoVariable("http://linked.earth/ontology/paleo_variables#TiO2", "TiO2")
    Ti_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Ti_Al", "Ti/Al")
    Uk37 = PaleoVariable("http://linked.earth/ontology/paleo_variables#Uk37", "Uk37")
    UK37 = PaleoVariable("http://linked.earth/ontology/paleo_variables#UK37", "UK37")
    Uk37_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#Uk37_", "Uk37’")
    V = PaleoVariable("http://linked.earth/ontology/paleo_variables#V", "V")
    V_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#V_Al", "V/Al")
    Y = PaleoVariable("http://linked.earth/ontology/paleo_variables#Y", "Y")
    Zn = PaleoVariable("http://linked.earth/ontology/paleo_variables#Zn", "Zn")
    Zr = PaleoVariable("http://linked.earth/ontology/paleo_variables#Zr", "Zr")
    Zr_Al = PaleoVariable("http://linked.earth/ontology/paleo_variables#Zr_Al", "Zr/Al")
    accumulation = PaleoVariable("http://linked.earth/ontology/paleo_variables#accumulation", "accumulation")
    age = PaleoVariable("http://linked.earth/ontology/paleo_variables#age", "age")
    age14C = PaleoVariable("http://linked.earth/ontology/paleo_variables#age14C", "age14C")
    ammonium = PaleoVariable("http://linked.earth/ontology/paleo_variables#ammonium", "ammonium")
    amps = PaleoVariable("http://linked.earth/ontology/paleo_variables#amps", "amps")
    aragonite = PaleoVariable("http://linked.earth/ontology/paleo_variables#aragonite", "aragonite")
    ash = PaleoVariable("http://linked.earth/ontology/paleo_variables#ash", "ash")
    boron = PaleoVariable("http://linked.earth/ontology/paleo_variables#boron", "boron")
    brGDGT_IIIa = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIa", "brGDGT-IIIa")
    brGDGT_Id = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-Id", "brGDGT-Id")
    brGDGT_IIIa_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIa_", "brGDGT-IIIa’")
    brGDGT_IIIb = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIb", "brGDGT-IIIb")
    brGDGT_IIIb_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIb_", "brGDGT-IIIb’")
    brGDGT_IIIc = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIc", "brGDGT-IIIc")
    brGDGT_IIIc_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIIc_", "brGDGT-IIIc’")
    brGDGT_IIa = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIa", "brGDGT-IIa")
    brGDGT_IIa_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIa_", "brGDGT-IIa’")
    brGDGT_IIb = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIb", "brGDGT-IIb")
    brGDGT_IIb_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIb_", "brGDGT-IIb’")
    brGDGT_IIc = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIc", "brGDGT-IIc")
    brGDGT_IIc_ = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-IIc_", "brGDGT-IIc’")
    brGDGT_Ia = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-Ia", "brGDGT-Ia")
    brGDGT_Ib = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-Ib", "brGDGT-Ib")
    brGDGT_Ic = PaleoVariable("http://linked.earth/ontology/paleo_variables#brGDGT-Ic", "brGDGT-Ic")
    sampleID = PaleoVariable("http://linked.earth/ontology/paleo_variables#sampleID", "sampleID")
    bubbleNumberDensity = PaleoVariable("http://linked.earth/ontology/paleo_variables#bubbleNumberDensity", "bubbleNumberDensity")
    bulkDensity = PaleoVariable("http://linked.earth/ontology/paleo_variables#bulkDensity", "bulkDensity")
    calcificationRate = PaleoVariable("http://linked.earth/ontology/paleo_variables#calcificationRate", "calcificationRate")
    calcite = PaleoVariable("http://linked.earth/ontology/paleo_variables#calcite", "calcite")
    carbon = PaleoVariable("http://linked.earth/ontology/paleo_variables#carbon", "carbon")
    carbonate = PaleoVariable("http://linked.earth/ontology/paleo_variables#carbonate", "carbonate")
    charcoal = PaleoVariable("http://linked.earth/ontology/paleo_variables#charcoal", "charcoal")
    chloride = PaleoVariable("http://linked.earth/ontology/paleo_variables#chloride", "chloride")
    circulationIndex = PaleoVariable("http://linked.earth/ontology/paleo_variables#circulationIndex", "circulationIndex")
    clay = PaleoVariable("http://linked.earth/ontology/paleo_variables#clay", "clay")
    cluster = PaleoVariable("http://linked.earth/ontology/paleo_variables#cluster", "cluster")
    index = PaleoVariable("http://linked.earth/ontology/paleo_variables#index", "index")
    composite = PaleoVariable("http://linked.earth/ontology/paleo_variables#composite", "composite")
    concentration = PaleoVariable("http://linked.earth/ontology/paleo_variables#concentration", "concentration")
    core = PaleoVariable("http://linked.earth/ontology/paleo_variables#core", "core")
    correction = PaleoVariable("http://linked.earth/ontology/paleo_variables#correction", "correction")
    correlationCoefficient = PaleoVariable("http://linked.earth/ontology/paleo_variables#correlationCoefficient", "correlationCoefficient")
    sampleCount = PaleoVariable("http://linked.earth/ontology/paleo_variables#sampleCount", "sampleCount")
    count = PaleoVariable("http://linked.earth/ontology/paleo_variables#count", "count")
    d13C = PaleoVariable("http://linked.earth/ontology/paleo_variables#d13C", "d13C")
    d15N = PaleoVariable("http://linked.earth/ontology/paleo_variables#d15N", "d15N")
    d18O = PaleoVariable("http://linked.earth/ontology/paleo_variables#d18O", "d18O")
    d2H = PaleoVariable("http://linked.earth/ontology/paleo_variables#d2H", "d2H")
    d2HUncertaintyHigh80 = PaleoVariable("http://linked.earth/ontology/paleo_variables#d2HUncertaintyHigh80", "d2HUncertaintyHigh80")
    d2HUncertaintyLow80 = PaleoVariable("http://linked.earth/ontology/paleo_variables#d2HUncertaintyLow80", "d2HUncertaintyLow80")
    deleteMe = PaleoVariable("http://linked.earth/ontology/paleo_variables#deleteMe", "deleteMe")
    needsToBeChanged = PaleoVariable("http://linked.earth/ontology/paleo_variables#needsToBeChanged", "needsToBeChanged")
    deltaRelativeHumidity = PaleoVariable("http://linked.earth/ontology/paleo_variables#deltaRelativeHumidity", "deltaRelativeHumidity")
    deltaTemperature = PaleoVariable("http://linked.earth/ontology/paleo_variables#deltaTemperature", "deltaTemperature")
    density = PaleoVariable("http://linked.earth/ontology/paleo_variables#density", "density")
    depth = PaleoVariable("http://linked.earth/ontology/paleo_variables#depth", "depth")
    depthBottom = PaleoVariable("http://linked.earth/ontology/paleo_variables#depthBottom", "depthBottom")
    depthTop = PaleoVariable("http://linked.earth/ontology/paleo_variables#depthTop", "depthTop")
    deuteriumExcess = PaleoVariable("http://linked.earth/ontology/paleo_variables#deuteriumExcess", "deuteriumExcess")
    diatom = PaleoVariable("http://linked.earth/ontology/paleo_variables#diatom", "diatom")
    diatomCount = PaleoVariable("http://linked.earth/ontology/paleo_variables#diatomCount", "diatomCount")
    dinocyst = PaleoVariable("http://linked.earth/ontology/paleo_variables#dinocyst", "dinocyst")
    dolomite = PaleoVariable("http://linked.earth/ontology/paleo_variables#dolomite", "dolomite")
    dryBulkDensity = PaleoVariable("http://linked.earth/ontology/paleo_variables#dryBulkDensity", "dryBulkDensity")
    duration = PaleoVariable("http://linked.earth/ontology/paleo_variables#duration", "duration")
    dust = PaleoVariable("http://linked.earth/ontology/paleo_variables#dust", "dust")
    effectivePrecipitation = PaleoVariable("http://linked.earth/ontology/paleo_variables#effectivePrecipitation", "effectivePrecipitation")
    elevation = PaleoVariable("http://linked.earth/ontology/paleo_variables#elevation", "elevation")
    zscore = PaleoVariable("http://linked.earth/ontology/paleo_variables#zscore", "zscore")
    epsilonC28C22 = PaleoVariable("http://linked.earth/ontology/paleo_variables#epsilonC28C22", "epsilonC28C22")
    epsilonC28C24 = PaleoVariable("http://linked.earth/ontology/paleo_variables#epsilonC28C24", "epsilonC28C24")
    epsilonC29C23 = PaleoVariable("http://linked.earth/ontology/paleo_variables#epsilonC29C23", "epsilonC29C23")
    equilibriumLineAltitude = PaleoVariable("http://linked.earth/ontology/paleo_variables#equilibriumLineAltitude", "equilibriumLineAltitude")
    event = PaleoVariable("http://linked.earth/ontology/paleo_variables#event", "event")
    eventLayer = PaleoVariable("http://linked.earth/ontology/paleo_variables#eventLayer", "eventLayer")
    facies = PaleoVariable("http://linked.earth/ontology/paleo_variables#facies", "facies")
    feldspar = PaleoVariable("http://linked.earth/ontology/paleo_variables#feldspar", "feldspar")
    flood = PaleoVariable("http://linked.earth/ontology/paleo_variables#flood", "flood")
    fluorine = PaleoVariable("http://linked.earth/ontology/paleo_variables#fluorine", "fluorine")
    foraminifera = PaleoVariable("http://linked.earth/ontology/paleo_variables#foraminifera", "foraminifera")
    gamma = PaleoVariable("http://linked.earth/ontology/paleo_variables#gamma", "gamma")
    glacierCoverage = PaleoVariable("http://linked.earth/ontology/paleo_variables#glacierCoverage", "glacierCoverage")
    globigerinoidesRuber = PaleoVariable("http://linked.earth/ontology/paleo_variables#globigerinoidesRuber", "globigerinoidesRuber")
    grainSize = PaleoVariable("http://linked.earth/ontology/paleo_variables#grainSize", "grainSize")
    lithics = PaleoVariable("http://linked.earth/ontology/paleo_variables#lithics", "lithics")
    grayscale = PaleoVariable("http://linked.earth/ontology/paleo_variables#grayscale", "grayscale")
    growing_degree_days = PaleoVariable("http://linked.earth/ontology/paleo_variables#growing_degree_days", "growing degree days")
    growthRate = PaleoVariable("http://linked.earth/ontology/paleo_variables#growthRate", "growthRate")
    hasGap = PaleoVariable("http://linked.earth/ontology/paleo_variables#hasGap", "hasGap")
    hasHiatus = PaleoVariable("http://linked.earth/ontology/paleo_variables#hasHiatus", "hasHiatus")
    hole = PaleoVariable("http://linked.earth/ontology/paleo_variables#hole", "hole")
    humidificationIndex = PaleoVariable("http://linked.earth/ontology/paleo_variables#humidificationIndex", "humidificationIndex")
    iceMelt = PaleoVariable("http://linked.earth/ontology/paleo_variables#iceMelt", "iceMelt")
    iceRaftedDebris = PaleoVariable("http://linked.earth/ontology/paleo_variables#iceRaftedDebris", "iceRaftedDebris")
    inc_coh = PaleoVariable("http://linked.earth/ontology/paleo_variables#inc_coh", "inc/coh")
    isReliable = PaleoVariable("http://linked.earth/ontology/paleo_variables#isReliable", "isReliable")
    lakeArea = PaleoVariable("http://linked.earth/ontology/paleo_variables#lakeArea", "lakeArea")
    lakeLevel = PaleoVariable("http://linked.earth/ontology/paleo_variables#lakeLevel", "lakeLevel")
    lakeTrend = PaleoVariable("http://linked.earth/ontology/paleo_variables#lakeTrend", "lakeTrend")
    lakeVolume = PaleoVariable("http://linked.earth/ontology/paleo_variables#lakeVolume", "lakeVolume")
    landscapeCover = PaleoVariable("http://linked.earth/ontology/paleo_variables#landscapeCover", "landscapeCover")
    percent = PaleoVariable("http://linked.earth/ontology/paleo_variables#percent", "percent")
    latitude = PaleoVariable("http://linked.earth/ontology/paleo_variables#latitude", "latitude")
    layerThickness = PaleoVariable("http://linked.earth/ontology/paleo_variables#layerThickness", "layerThickness")
    longitude = PaleoVariable("http://linked.earth/ontology/paleo_variables#longitude", "longitude")
    material = PaleoVariable("http://linked.earth/ontology/paleo_variables#material", "material")
    mineralogy = PaleoVariable("http://linked.earth/ontology/paleo_variables#mineralogy", "mineralogy")
    sulfur = PaleoVariable("http://linked.earth/ontology/paleo_variables#sulfur", "sulfur")
    needsToBeSplitIntoMultipleColumns = PaleoVariable("http://linked.earth/ontology/paleo_variables#needsToBeSplitIntoMultipleColumns", "needsToBeSplitIntoMultipleColumns")
    nitrogen = PaleoVariable("http://linked.earth/ontology/paleo_variables#nitrogen", "nitrogen")
    notes = PaleoVariable("http://linked.earth/ontology/paleo_variables#notes", "notes")
    organicMatter = PaleoVariable("http://linked.earth/ontology/paleo_variables#organicMatter", "organicMatter")
    organicNitrogen = PaleoVariable("http://linked.earth/ontology/paleo_variables#organicNitrogen", "organicNitrogen")
    oxygen = PaleoVariable("http://linked.earth/ontology/paleo_variables#oxygen", "oxygen")
    pH = PaleoVariable("http://linked.earth/ontology/paleo_variables#pH", "pH")
    peat = PaleoVariable("http://linked.earth/ontology/paleo_variables#peat", "peat")
    phosphorus = PaleoVariable("http://linked.earth/ontology/paleo_variables#phosphorus", "phosphorus")
    potassium = PaleoVariable("http://linked.earth/ontology/paleo_variables#potassium", "potassium")
    precipitation = PaleoVariable("http://linked.earth/ontology/paleo_variables#precipitation", "precipitation")
    productivity = PaleoVariable("http://linked.earth/ontology/paleo_variables#productivity", "productivity")
    pyrite = PaleoVariable("http://linked.earth/ontology/paleo_variables#pyrite", "pyrite")
    quartz = PaleoVariable("http://linked.earth/ontology/paleo_variables#quartz", "quartz")
    reflectance = PaleoVariable("http://linked.earth/ontology/paleo_variables#reflectance", "reflectance")
    relativeHumidity = PaleoVariable("http://linked.earth/ontology/paleo_variables#relativeHumidity", "relativeHumidity")
    residualChronology = PaleoVariable("http://linked.earth/ontology/paleo_variables#residualChronology", "residualChronology")
    ringWidth = PaleoVariable("http://linked.earth/ontology/paleo_variables#ringWidth", "ringWidth")
    sand = PaleoVariable("http://linked.earth/ontology/paleo_variables#sand", "sand")
    seaIce = PaleoVariable("http://linked.earth/ontology/paleo_variables#seaIce", "seaIce")
    section = PaleoVariable("http://linked.earth/ontology/paleo_variables#section", "section")
    sedimentDry = PaleoVariable("http://linked.earth/ontology/paleo_variables#sedimentDry", "sedimentDry")
    sedimentationRate = PaleoVariable("http://linked.earth/ontology/paleo_variables#sedimentationRate", "sedimentationRate")
    segmentLength = PaleoVariable("http://linked.earth/ontology/paleo_variables#segmentLength", "segmentLength")
    sequence = PaleoVariable("http://linked.earth/ontology/paleo_variables#sequence", "sequence")
    silt = PaleoVariable("http://linked.earth/ontology/paleo_variables#silt", "silt")
    site = PaleoVariable("http://linked.earth/ontology/paleo_variables#site", "site")
    siteCount = PaleoVariable("http://linked.earth/ontology/paleo_variables#siteCount", "siteCount")
    sodium = PaleoVariable("http://linked.earth/ontology/paleo_variables#sodium", "sodium")
    solarIrradiance = PaleoVariable("http://linked.earth/ontology/paleo_variables#solarIrradiance", "solarIrradiance")
    streamflow = PaleoVariable("http://linked.earth/ontology/paleo_variables#streamflow", "streamflow")
    temperature = PaleoVariable("http://linked.earth/ontology/paleo_variables#temperature", "temperature")
    thickness = PaleoVariable("http://linked.earth/ontology/paleo_variables#thickness", "thickness")
    totalCarbon = PaleoVariable("http://linked.earth/ontology/paleo_variables#totalCarbon", "totalCarbon")
    totalNitrogen = PaleoVariable("http://linked.earth/ontology/paleo_variables#totalNitrogen", "totalNitrogen")
    totalPollen = PaleoVariable("http://linked.earth/ontology/paleo_variables#totalPollen", "totalPollen")
    treeCover = PaleoVariable("http://linked.earth/ontology/paleo_variables#treeCover", "treeCover")
    uncertainty = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertainty", "uncertainty")
    uncertainty1s = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertainty1s", "uncertainty1s")
    uncertainty2s = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertainty2s", "uncertainty2s")
    uncertaintyHigh = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyHigh", "uncertaintyHigh")
    uncertaintyHigh1s = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyHigh1s", "uncertaintyHigh1s")
    uncertaintyLow95 = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyLow95", "uncertaintyLow95")
    uncertaintyHigh50 = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyHigh50", "uncertaintyHigh50")
    uncertaintyHigh90 = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyHigh90", "uncertaintyHigh90")
    uncertaintyHigh95 = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyHigh95", "uncertaintyHigh95")
    uncertaintyLow = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyLow", "uncertaintyLow")
    uncertaintyLow1s = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyLow1s", "uncertaintyLow1s")
    uncertaintyLow90 = PaleoVariable("http://linked.earth/ontology/paleo_variables#uncertaintyLow90", "uncertaintyLow90")
    upwelling = PaleoVariable("http://linked.earth/ontology/paleo_variables#upwelling", "upwelling")
    uranium = PaleoVariable("http://linked.earth/ontology/paleo_variables#uranium", "uranium")
    varveThickness = PaleoVariable("http://linked.earth/ontology/paleo_variables#varveThickness", "varveThickness")
    volume = PaleoVariable("http://linked.earth/ontology/paleo_variables#volume", "volume")
    waterContent = PaleoVariable("http://linked.earth/ontology/paleo_variables#waterContent", "waterContent")
    waterTableDepth = PaleoVariable("http://linked.earth/ontology/paleo_variables#waterTableDepth", "waterTableDepth")
    wetBulkDensity = PaleoVariable("http://linked.earth/ontology/paleo_variables#wetBulkDensity", "wetBulkDensity")
    year = PaleoVariable("http://linked.earth/ontology/paleo_variables#year", "year")